@@ -1,39 +1,94 @@
-import os.path
-import platform
 import logging
+import os.path
 import sys
-import traceback
-from typing import TYPE_CHECKING, Optional, cast
+import time
+from logging import getLogger
+from typing import TYPE_CHECKING, List, Optional, cast
+
+from thonny.common import is_private_python, is_virtual_executable
+
+_last_module_count = 0
+_last_modules = set()
+
+_last_time = time.time()
+
+
+logger = getLogger(__name__)
+
+
+def report_time(label: str) -> None:
+    """
+    Method for finding unwarranted imports and delays.
+    """
+    # return
+
+    global _last_time, _last_module_count, _last_modules
+
+    log_modules = True
+
+    t = time.time()
+    mod_count = len(sys.modules)
+    mod_delta = mod_count - _last_module_count
+    if mod_delta > 0:
+        mod_info = f"(+{mod_count - _last_module_count} modules)"
+    else:
+        mod_info = ""
+    logger.info("TIME/MODS %s %s %s", f"{t - _last_time:.3f}", label, mod_info)
+
+    if log_modules and mod_delta > 0:
+        current_modules = set(sys.modules.keys())
+        logger.info("NEW MODS %s", list(sorted(current_modules - _last_modules)))
+        _last_modules = current_modules
+
+    _last_time = t
+    _last_module_count = mod_count
+
+
+report_time("After defining report_time")
+
 
 SINGLE_INSTANCE_DEFAULT = True
+BACKEND_LOG_MARKER = "Thonny's backend.log"
+
+
+def _get_known_folder(ID):
+    # http://stackoverflow.com/a/3859336/261181
+    # http://www.installmate.com/support/im9/using/symbols/functions/csidls.htm
+    import ctypes.wintypes
+
+    SHGFP_TYPE_CURRENT = 0
+    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
+    ctypes.windll.shell32.SHGetFolderPathW(0, ID, 0, SHGFP_TYPE_CURRENT, buf)
+    assert buf.value
+    return buf.value
+
+
+def _get_roaming_appdata_dir():
+    return _get_known_folder(26)
+
+
+def _get_local_appdata_dir():
+    return _get_known_folder(28)
 
 
 def _compute_thonny_user_dir():
     if os.environ.get("THONNY_USER_DIR", ""):
         return os.path.expanduser(os.environ["THONNY_USER_DIR"])
     elif is_portable():
-        if platform.system() == "Windows":
+        if sys.platform == "win32":
             root_dir = os.path.dirname(sys.executable)
-        elif platform.system() == "Darwin":
+        elif sys.platform == "darwin":
             root_dir = os.path.join(
                 os.path.dirname(sys.executable), "..", "..", "..", "..", "..", ".."
             )
         else:
             root_dir = os.path.join(os.path.dirname(sys.executable), "..")
         return os.path.normpath(os.path.abspath(os.path.join(root_dir, "user_data")))
-    elif (
-        hasattr(sys, "base_prefix")
-        and sys.base_prefix != sys.prefix
-        or hasattr(sys, "real_prefix")
-        and getattr(sys, "real_prefix") != sys.prefix
-    ):
-        # we're in a virtualenv or venv
+    elif is_virtual_executable(sys.executable) and not is_private_python(sys.executable):
         return os.path.join(sys.prefix, ".thonny")
-    elif platform.system() == "Windows":
-        from thonny import misc_utils
-
-        return os.path.join(misc_utils.get_roaming_appdata_dir(), "Thonny")
-    elif platform.system() == "Darwin":
+    elif sys.platform == "win32":
+        return os.path.join(_get_roaming_appdata_dir(), "Thonny")
+    elif sys.platform == "darwin":
         return os.path.expanduser("~/Library/Thonny")
     else:
         # https://specifications.freedesktop.org/basedir-spec/latest/ar01s02.html
@@ -72,7 +127,7 @@ def is_portable():
 
     # ... or it becomes implicitly portable if it's on a removable drive
     abs_location = os.path.abspath(__file__)
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         drive = os.path.splitdrive(abs_location)[0]
         if drive.endswith(":"):
             from ctypes import windll
@@ -80,7 +135,7 @@ def is_portable():
             return windll.kernel32.GetDriveTypeW(drive) == 2  # @UndefinedVariable
         else:
             return False
-    elif platform.system() == "Darwin":
+    elif sys.platform == "darwin":
         # not exact heuristics
         return abs_location.startswith("/Volumes/")
     else:
@@ -110,10 +165,8 @@ def get_ipc_file_path():
     if _IPC_FILE:
         return _IPC_FILE
 
-    from thonny import misc_utils
-
-    if platform.system() == "Windows":
-        base_dir = misc_utils.get_local_appdata_dir()
+    if sys.platform == "win32":
+        base_dir = _get_local_appdata_dir()
     else:
         base_dir = os.environ.get("XDG_RUNTIME_DIR")
         if not base_dir or not os.path.exists(base_dir):
@@ -132,7 +185,7 @@ def get_ipc_file_path():
     ipc_dir = os.path.join(base_dir, "thonny-%s" % username)
     os.makedirs(ipc_dir, exist_ok=True)
 
-    if not platform.system() == "Windows":
+    if not sys.platform == "win32":
         os.chmod(ipc_dir, 0o700)
 
     _IPC_FILE = os.path.join(ipc_dir, "ipc.sock")
@@ -157,7 +210,6 @@ def _check_welcome():
 
 def launch():
     import runpy
-    import socket
 
     if sys.executable.endswith("thonny.exe"):
         # otherwise some library may try to run its subprocess with thonny.exe
@@ -194,27 +246,19 @@ def launch():
         from thonny import workbench
 
         bench = workbench.Workbench()
-        try:
-            bench.mainloop()
-        except SystemExit:
-            bench.destroy()
+        bench.mainloop()
         return 0
 
-    except SystemExit as e:
-        from tkinter import messagebox, _default_root
-
-        messagebox.showerror("System exit", str(e), master=_default_root)
-        return -1
-
     except Exception:
-        from logging import exception
+        import tkinter as tk
         import traceback
+        from logging import exception
 
         exception("Internal launch or mainloop error")
         from thonny import ui_utils
 
         dlg = ui_utils.LongTextDialog("Internal error", traceback.format_exc())
-        ui_utils.show_dialog(dlg, get_workbench())
+        ui_utils.show_dialog(dlg, tk._default_root)
         return -1
     finally:
         runner = get_runner()
@@ -308,7 +352,7 @@ def _create_client_socket():
 
     timeout = 2.0
 
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         with open(get_ipc_file_path(), "r") as fp:
             port = int(fp.readline().strip())
             secret = fp.readline().strip()
@@ -325,34 +369,82 @@ def _create_client_socket():
 
 
 def _configure_frontend_logging() -> None:
-    _configure_logging("frontend.log")
+    _configure_logging(get_frontend_log_file(), _choose_logging_level())
 
 
 def configure_backend_logging() -> None:
-    _configure_logging("backend.log", logging.WARNING)
+    _configure_logging(get_backend_log_file(), None)
 
 
-def _configure_logging(filename, console_level=None):
-    logFormatter = logging.Formatter("%(levelname)-7s %(name)s: %(message)s")
+def get_backend_log_file():
+    return os.path.join(THONNY_USER_DIR, "backend.log")
 
-    # NB! Don't mess with the root logger, because (CPython) backend runs user code
-    thonny_root_logger = logging.getLogger("thonny")
-    thonny_root_logger.setLevel(_choose_logging_level())
-    thonny_root_logger.propagate = False  # otherwise it will be also reported by IDE-s root logger
 
-    log_file = os.path.join(THONNY_USER_DIR, filename)
+def get_frontend_log_file():
+    return os.path.join(THONNY_USER_DIR, "frontend.log")
+
+
+def _get_orig_argv() -> Optional[List[str]]:
+    try:
+        from sys import orig_argv  # since 3.10
+
+        return sys.orig_argv
+    except ImportError:
+        # https://stackoverflow.com/a/57914236/261181
+        import ctypes
+
+        argc = ctypes.c_int()
+        argv = ctypes.POINTER(ctypes.c_wchar_p if sys.version_info >= (3,) else ctypes.c_char_p)()
+        try:
+            ctypes.pythonapi.Py_GetArgcArgv(ctypes.byref(argc), ctypes.byref(argv))
+        except AttributeError:
+            # See https://github.com/thonny/thonny/issues/2206
+            # and https://bugs.python.org/issue40910
+            # This symbol is not available in thonny.exe built agains Python 3.8
+            return None
+
+        # Ctypes are weird. They can't be used in list comprehensions, you can't use `in` with them, and you can't
+        # use a for-each loop on them. We have to do an old-school for-i loop.
+        arguments = list()
+        for i in range(argc.value):
+            arguments.append(argv[i])
+
+        return arguments
+
+
+def _configure_logging(log_file, console_level=None):
+    logFormatter = logging.Formatter(
+        "%(asctime)s.%(msecs)03d %(levelname)-7s %(name)s: %(message)s", "%H:%M:%S"
+    )
+
     file_handler = logging.FileHandler(log_file, encoding="UTF-8", mode="w")
     file_handler.setFormatter(logFormatter)
-    thonny_root_logger.addHandler(file_handler)
 
-    console_handler = logging.StreamHandler(sys.stdout)
-    console_handler.setFormatter(logFormatter)
-    if console_level is None:
-        console_level = _choose_logging_level()
-    console_handler.setLevel(console_level)
-    thonny_root_logger.addHandler(console_handler)
+    main_logger = logging.getLogger("thonny")
+    contrib_logger = logging.getLogger("thonnycontrib")
+    pipkin_logger = logging.getLogger("pipkin")
 
-    thonny_root_logger.info("Thonny version: %s", get_version())
+    # NB! Don't mess with the main root logger, because (CPython) backend runs user code
+    for logger in [main_logger, contrib_logger, pipkin_logger]:
+        logger.setLevel(_choose_logging_level())
+        logger.propagate = False  # otherwise it will be also reported by IDE-s root logger
+        logger.addHandler(file_handler)
+
+    if console_level is not None:
+        console_handler = logging.StreamHandler(sys.stdout)
+        console_handler.setFormatter(logFormatter)
+        console_handler.setLevel(console_level)
+        for logger in [main_logger, contrib_logger]:
+            logger.addHandler(console_handler)
+
+    # Log most important info as soon as possible
+    main_logger.info("Thonny version: %s", get_version())
+    main_logger.info("cwd: %s", os.getcwd())
+    main_logger.info("original argv: %s", _get_orig_argv())
+    main_logger.info("sys.executable: %s", sys.executable)
+    main_logger.info("sys.argv: %s", sys.argv)
+    main_logger.info("sys.path: %s", sys.path)
+    main_logger.info("sys.flags: %s", sys.flags)
 
     import faulthandler
 
@@ -360,6 +452,16 @@ def _configure_logging(filename, console_level=None):
     faulthandler.enable(fault_out)
 
 
+def get_user_base_directory_for_plugins() -> str:
+    return os.path.join(THONNY_USER_DIR, "plugins")
+
+
+def get_sys_path_directory_containg_plugins() -> str:
+    from thonny.misc_utils import get_user_site_packages_dir_for_base
+
+    return get_user_site_packages_dir_for_base(get_user_base_directory_for_plugins())
+
+
 def set_dpi_aware():
     # https://stackoverflow.com/questions/36134072/setprocessdpiaware-seems-not-to-work-under-windows-10
     # https://bugs.python.org/issue33656
@@ -428,3 +530,6 @@ def get_runner() -> "Runner":
 
 def get_shell() -> "ShellView":
     return cast("ShellView", get_workbench().get_view("ShellView"))
+
+
+report_time("After loading thonny module")
