@@ -1,12 +1,12 @@
 import ast
 import datetime
-import logging
 import os.path
 import subprocess
 import sys
 import textwrap
 import tkinter as tk
 from collections import namedtuple
+from logging import getLogger
 from tkinter import messagebox, ttk
 from typing import Dict  # pylint disable=unused-import
 from typing import List  # pylint disable=unused-import
@@ -18,12 +18,17 @@ from typing import Iterable
 
 import thonny
 from thonny import get_runner, get_workbench, rst_utils, tktextext, ui_utils
-from thonny.common import ToplevelResponse, read_source
+from thonny.common import (
+    REPL_PSEUDO_FILENAME,
+    STRING_PSEUDO_FILENAME,
+    ToplevelResponse,
+    read_source,
+)
 from thonny.languages import tr
 from thonny.misc_utils import levenshtein_damerau_distance, running_on_mac_os
-from thonny.ui_utils import CommonDialog, scrollbar_style
+from thonny.ui_utils import CommonDialog, get_hyperlink_cursor, scrollbar_style
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 Suggestion = namedtuple("Suggestion", ["symbol", "title", "body", "relevance"])
 
@@ -105,14 +110,15 @@ class AssistantView(tktextext.TextFrame):
 
         self._clear()
 
-        from thonny.plugins.cpython import CPythonProxy
+        from thonny.plugins.cpython_frontend import LocalCPythonProxy
 
-        if not isinstance(get_runner().get_backend_proxy(), CPythonProxy):
+        if not isinstance(get_runner().get_backend_proxy(), LocalCPythonProxy):
             # TODO: add some support for MicroPython as well
             return
 
         # prepare for snapshot
-        key = msg.get("filename", "<pyshell>")
+        # TODO: should distinguish between <string> and <stdin> ?
+        key = msg.get("filename", STRING_PSEUDO_FILENAME)
         self._current_snapshot = {
             "timestamp": datetime.datetime.now().isoformat()[:19],
             "main_file_path": key,
@@ -645,7 +651,7 @@ class FeedbackDialog(CommonDialog):
             main_frame,
             text="(Preview the data to be sent)",
             style="Url.TLabel",
-            cursor="hand2",
+            cursor=get_hyperlink_cursor(),
             font=url_font,
         )
         preview_link.bind("<1>", self._preview_submission_data, True)
@@ -669,7 +675,7 @@ class FeedbackDialog(CommonDialog):
         self._populate_tree()
 
     def _happened_in_shell(self):
-        return self.main_file_path is None or self.main_file_path.lower() == "<pyshell>"
+        return self.main_file_path is None or self.main_file_path == REPL_PSEUDO_FILENAME
 
     def _populate_tree(self):
         groups = {}
@@ -772,14 +778,14 @@ class FeedbackDialog(CommonDialog):
 
             submission["mypy_version"] = str(mypy.version.__version__)
         except ImportError:
-            logging.exception("Could not get MyPy version")
+            logger.exception("Could not get MyPy version")
 
         try:
             import pylint
 
             submission["pylint_version"] = str(pylint.__version__)
         except ImportError:
-            logging.exception("Could not get Pylint version")
+            logger.exception("Could not get Pylint version")
 
         if self.include_snapshots_var.get():
             submission["snapshots"] = self.snapshots
