@@ -1,23 +1,28 @@
 import datetime
-import logging
 import os.path
+import shutil
+import stat
 import subprocess
 import time
 import tkinter as tk
-from tkinter import messagebox, ttk
+from logging import getLogger
+from tkinter import messagebox, simpledialog, ttk
+from typing import Optional
 
 from thonny import get_runner, get_workbench, misc_utils, tktextext
-from thonny.common import InlineCommand, get_dirs_children_info
+from thonny.common import InlineCommand, UserError, get_dirs_children_info
 from thonny.languages import tr
-from thonny.misc_utils import running_on_windows, sizeof_fmt, running_on_mac_os
+from thonny.misc_utils import running_on_mac_os, running_on_windows, sizeof_fmt
 from thonny.ui_utils import (
     CommonDialog,
+    ask_one_from_choices,
+    ask_string,
     create_string_var,
+    ems_to_pixels,
+    get_hyperlink_cursor,
     lookup_style_option,
     scrollbar_style,
     show_dialog,
-    ask_string,
-    ask_one_from_choices,
 )
 
 _dummy_node_text = "..."
@@ -27,7 +32,7 @@ ROOT_NODE_ID = ""
 
 HIDDEN_FILES_OPTION = "file.show_hidden_files"
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 
 class BaseFileBrowser(ttk.Frame):
@@ -73,6 +78,8 @@ class BaseFileBrowser(ttk.Frame):
         self.tree.bind("<Double-Button-1>", self.on_double_click, True)
         self.tree.bind("<<TreeviewOpen>>", self.on_open_node)
 
+        self.copypaste = None
+
         wb = get_workbench()
         self.folder_icon = wb.get_image("folder")
         self.python_file_icon = wb.get_image("python-icon")
@@ -110,8 +117,8 @@ class BaseFileBrowser(ttk.Frame):
             height=1,
             font="TkDefaultFont",
             wrap="word",
-            padx=6,
-            pady=5,
+            padx=ems_to_pixels(0.6),
+            pady=ems_to_pixels(0.5),
             insertwidth=0,
             highlightthickness=0,
             background=lookup_style_option("ViewToolbar.TFrame", "background"),
@@ -137,7 +144,7 @@ class BaseFileBrowser(ttk.Frame):
                 self.path_bar.tag_add("underline", range_start, range_end)
 
         def dir_tag_enter(event):
-            self.path_bar.config(cursor="hand2")
+            self.path_bar.config(cursor=get_hyperlink_cursor())
 
         def dir_tag_leave(event):
             self.path_bar.config(cursor="")
@@ -177,6 +184,9 @@ class BaseFileBrowser(ttk.Frame):
         self.tree.set_children("")
         self.current_focus = None
 
+    def path_exists(self, path: str) -> Optional[bool]:
+        return None
+
     def request_focus_into(self, path):
         return self.focus_into(path)
 
@@ -229,6 +239,10 @@ class BaseFileBrowser(ttk.Frame):
 
     def on_open_node(self, event):
         node_id = self.get_selected_node()
+        if self.get_selected_kind() == "file":
+            # can happen in Windows when pressing ENTER on file
+            return "break"
+
         path = self.tree.set(node_id, "path")
         if path:  # and path not in self._cached_child_data:
             self.render_children_from_cache(node_id)
@@ -377,14 +391,17 @@ class BaseFileBrowser(ttk.Frame):
                 self.tree.selection_set(child_id)
                 return
 
-            if self.tree.item(child_id, "open"):
+            if self._is_open_dir_node(child_id):
                 self.select_path_if_visible(path, child_id)
 
+    def _is_open_dir_node(self, node_id) -> bool:
+        # In Windows a node may get open=True simply by pressing ENTER on it
+        return self.tree.item(node_id, "open") and self.tree.set(node_id, "kind") != "file"
+
     def get_open_paths(self, node_id=ROOT_NODE_ID):
         if self.tree.set(node_id, "kind") == "file":
             return set()
-
-        elif node_id == ROOT_NODE_ID or self.tree.item(node_id, "open"):
+        elif node_id == ROOT_NODE_ID or self._is_open_dir_node(node_id):
             result = {self.tree.set(node_id, "path")}
             for child_id in self.tree.get_children(node_id):
                 result.update(self.get_open_paths(child_id))
@@ -405,6 +422,12 @@ class BaseFileBrowser(ttk.Frame):
         """This node is supposed to be a directory and
         its contents needs to be shown and/or refreshed"""
         path = self.tree.set(node_id, "path")
+        kind = self.tree.set(node_id, "kind")
+        if kind == "file":
+            logger.warning("File %r is treated as dir", path)
+            return
+
+        logger.debug("Rendering %r from cache", path)
 
         if path not in self._cached_child_data:
             self.request_dirs_child_data(node_id, self.get_open_paths() | {path})
@@ -468,7 +491,7 @@ class BaseFileBrowser(ttk.Frame):
 
             # recursively update open children
             for child_id in ids_sorted_by_name:
-                if self.tree.item(child_id, "open"):
+                if self._is_open_dir_node(child_id):
                     self.render_children_from_cache(child_id)
 
     def show_error(self, msg, node_id=""):
@@ -624,10 +647,6 @@ class BaseFileBrowser(ttk.Frame):
                 command=lambda: self.open_path_with_system_app(selected_path),
             )
 
-            hidden_files_label = (
-                tr("Hide hidden files") if show_hidden_files() else tr("Show hidden files")
-            )
-            self.menu.add_command(label=hidden_files_label, command=self.toggle_hidden_files)
         else:
             if selected_kind == "dir":
                 self.menu.add_command(
@@ -640,7 +659,7 @@ class BaseFileBrowser(ttk.Frame):
 
             if self.is_active_browser():
                 self.menu.add_command(
-                    label=tr("Open in system default app"),
+                    label=tr("Open in default external app"),
                     command=lambda: self.open_path_with_system_app(selected_path),
                 )
 
@@ -651,6 +670,11 @@ class BaseFileBrowser(ttk.Frame):
                         command=lambda: self.open_extension_dialog(ext),
                     )
 
+        hidden_files_label = (
+            tr("Hide hidden files") if show_hidden_files() else tr("Show hidden files")
+        )
+        self.menu.add_command(label=hidden_files_label, command=self.toggle_hidden_files)
+
     def toggle_hidden_files(self):
         get_workbench().set_option(
             HIDDEN_FILES_OPTION, not get_workbench().get_option(HIDDEN_FILES_OPTION)
@@ -690,18 +714,33 @@ class BaseFileBrowser(ttk.Frame):
         self.refresh_tree()
 
     def add_middle_menu_items(self, context):
-        if self.supports_trash():
-            if running_on_windows():
-                trash_label = tr("Move to Recycle Bin")
-            else:
-                trash_label = tr("Move to Trash")
-            self.menu.add_command(label=trash_label, command=self.move_to_trash)
-        else:
-            self.menu.add_command(label=tr("Delete"), command=self.delete)
+        if self.supports_new_file():
+            self.menu.add_command(label=tr("New file") + "...", command=self.create_new_file)
 
         if self.supports_directories():
             self.menu.add_command(label=tr("New directory") + "...", command=self.mkdir)
 
+        if self.supports_copypaste():
+            self.menu.add_command(label=tr("Cut"), command=self.cut_files)
+            self.menu.add_command(label=tr("Copy"), command=self.copy_files)
+            target = self.get_selected_file()
+            self.menu.add_command(label=tr("Paste"), command=self.paste_files)
+            if (
+                target is None
+                or not self.copypaste.has_selection()
+                or self.copypaste.conflicts(target)
+            ):
+                self.menu.entryconfig(tr("Paste"), state="disabled")
+
+        if self.supports_rename():
+            self.menu.add_command(label=tr("Rename"), command=self.rename_file)
+
+        if self.supports_trash():
+            trash_label = tr("Move to Trash")
+            self.menu.add_command(label=trash_label, command=self.move_to_trash)
+        else:
+            self.menu.add_command(label=tr("Delete"), command=self.delete)
+
     def add_last_menu_items(self, context):
         self.menu.add_command(label=tr("Properties"), command=self.show_properties)
         if context == "button":
@@ -772,22 +811,24 @@ class BaseFileBrowser(ttk.Frame):
 
         path = self.join(parent_path, name)
 
-        if name in self._cached_child_data[parent_path]:
-            # TODO: ignore case in windows
+        if self.path_exists(path):
             messagebox.showerror("Error", "The file '" + path + "' already exists", master=self)
             return self.create_new_file()
         else:
-            self.open_file(path)
+            self.create_new_file_editor(path)
 
         return path
 
+    def create_new_file_editor(self, path):
+        raise NotImplementedError()
+
     def delete(self):
         selection = self.get_selection_info(True)
         if not selection:
             return
 
         confirmation = "Are you sure want to delete %s?" % selection["description"]
-        confirmation += "\n\nNB! Recycle bin won't be used (no way to undelete)!"
+        confirmation += "\n\nNB! Trash bin won't be used (no way to undelete)!"
         if "dir" in selection["kinds"]:
             confirmation += "\n" + "Directories will be deleted with content."
 
@@ -804,17 +845,16 @@ class BaseFileBrowser(ttk.Frame):
         if not selection:
             return
 
-        trash = tr("Recycle Bin") if running_on_windows() else tr("Trash")
         if not messagebox.askokcancel(
-            tr("Moving to %s") % trash,
-            tr("Move %s to %s?") % (selection["description"], trash),
+            tr("Moving to Trash"),
+            tr("Move %s to Trash?") % selection["description"],
             icon="info",
             master=self,
         ):
             return
 
         self.perform_move_to_trash(
-            selection["paths"], tr("Moving %s to %s") % (selection["description"], trash)
+            selection["paths"], tr("Moving %s to Trash") % (selection["description"])
         )
         self.refresh_tree()
 
@@ -862,18 +902,152 @@ class BaseFileBrowser(ttk.Frame):
         ext = self.get_extension_from_name(name)
         return get_workbench().get_option(get_file_handler_conf_key(ext), "system") == "thonny"
 
+    def supports_new_file(self):
+        return False
+
+    def get_selected_file(self):
+        selection = self.get_selection_info(False)
+        if not selection or len(selection["paths"]) > 1:
+            return
+        return selection["paths"][0]
+
+    def supports_rename(self):
+        return False
+
+    def rename_file(self):
+        old_name = self.get_selected_file()
+        if old_name is None:
+            return
+        file_name = os.path.basename(old_name)
+
+        new_name = simpledialog.askstring(
+            tr("Rename '%s'") % file_name, tr("Enter new name"), initialvalue=file_name, parent=self
+        )
+        if new_name:
+            self.perform_rename(old_name, new_name)
+            self.refresh_tree()
+
+    def perform_rename(self, old_name, new_name):
+        raise Exception("overload this in subclass")
+
+    def supports_copypaste(self):
+        return self.copypaste
+
+    def cut_files(self):
+        assert self.copypaste
+        self.copypaste.cut()
+
+    def copy_files(self):
+        assert self.copypaste
+        self.copypaste.copy()
+
+    def paste_files(self):
+        assert self.copypaste
+        target = self.get_selected_file()
+        assert target
+        if os.path.isfile(target):
+            target = os.path.dirname(target)
+        self.copypaste.paste(target)
+
+
+class CopyPaste(object):
+    def __init__(self, filebrowser):
+        self.fb = filebrowser
+        self.reset()
+
+    def reset(self):
+        self.mode = None
+        self.paths = []
+
+    def get_selection_paths(self):
+        selection = self.fb.get_selection_info(True)
+        if selection:
+            return selection["paths"]
+        return []
+
+    def has_selection(self):
+        return len(self.paths) > 0
+
+    def dirname(self, path):
+        if os.path.isfile(path):
+            path = os.path.dirname(path)
+        return path
+
+    def _conflicts(self, target):
+        target = self.dirname(target) + os.sep
+        folder = set(map(lambda x: self.dirname(x) + os.sep, self.paths))
+
+        if target in folder:
+            return True
+
+        for p in self.paths:
+            p_isdir = os.path.isdir(p)
+            if p_isdir:
+                if target.startswith(p + os.sep):
+                    return True
+                if p.startswith(target):
+                    _conf = p.find(os.sep, len(target)) < 0
+                    if _conf:
+                        return True
+        return False
+
+    def conflicts(self, target):
+        conf = self._conflicts(target)
+        return conf
+
+    def cut(self):
+        self.paths = self.get_selection_paths()
+        self.mode = "X"
+
+    def copy(self):
+        self.paths = self.get_selection_paths()
+        self.mode = "C"
+
+    def paste(self, target):
+        # https://docs.python.org/3/library/shutil.html#shutil.move
+        # depends on os.rename semantics
+        try:
+            if self.conflicts(target):
+                messagebox.showerror(
+                    tr("Paste failed"), tr("There are conflicting folders"), parent=self
+                )
+                return
+
+            for f in self.paths:
+                isfile = os.path.isfile(f)
+                if self.mode == "C":
+                    if isfile:
+                        shutil.copy2(f, target)
+                    else:
+                        folder_target = os.path.join(target, os.path.basename(f))
+                        shutil.copytree(f, folder_target, dirs_exist_ok=True)
+                elif self.mode == "X":
+                    dest = shutil.move(f, target)
+                else:
+                    raise Exception("unsupported mode")
+
+        finally:
+            self.reset()
+
 
 class BaseLocalFileBrowser(BaseFileBrowser):
     def __init__(self, master, show_expand_buttons=True):
         super().__init__(master, show_expand_buttons=show_expand_buttons)
         get_workbench().bind("WindowFocusIn", self.on_window_focus_in, True)
         get_workbench().bind("LocalFileOperation", self.on_local_file_operation, True)
+        self.copypaste = CopyPaste(self)
 
     def destroy(self):
         super().destroy()
         get_workbench().unbind("WindowFocusIn", self.on_window_focus_in)
         get_workbench().unbind("LocalFileOperation", self.on_local_file_operation)
 
+    def path_exists(self, path: str) -> Optional[bool]:
+        return os.path.exists(path)
+
+    def create_new_file_editor(self, path):
+        get_workbench().get_editor_notebook().open_new_file(path)
+
     def request_dirs_child_data(self, node_id, paths):
         self.cache_dirs_child_data(get_dirs_children_info(paths, show_hidden_files()))
         self.render_children_from_cache(node_id)
@@ -900,10 +1074,10 @@ class BaseLocalFileBrowser(BaseFileBrowser):
         try:
             open_with_default_app(path)
         except Exception as e:
-            logger.error("Could not open %r in system app", path, exc_info=e)
+            logger.error("Could not open %r in external. app", path, exc_info=e)
             messagebox.showerror(
                 "Error",
-                "Could not open '%s' in system app\nError: %s" % (path, e),
+                "Could not open '%s' in external app\nError: %s" % (path, e),
                 parent=self.winfo_toplevel(),
             )
 
@@ -948,6 +1122,25 @@ class BaseLocalFileBrowser(BaseFileBrowser):
         except ImportError:
             return False
 
+    def supports_new_file(self):
+        return True
+
+    def supports_rename(self):
+        return self.get_selected_file()
+
+    def perform_rename(self, old_name, new_name):
+        basepath = os.path.dirname(old_name)
+        full_path = os.path.join(basepath, new_name)
+        logger.debug("rename %s to %s" % (old_name, full_path))
+
+        if old_name == full_path:
+            return
+
+        if os.path.exists(full_path):
+            raise UserError(tr("File already exists"))
+
+        os.rename(old_name, full_path)
+
 
 class BaseRemoteFileBrowser(BaseFileBrowser):
     def __init__(self, master, show_expand_buttons=True):
@@ -971,6 +1164,9 @@ class BaseRemoteFileBrowser(BaseFileBrowser):
 
         return "Back-end"
 
+    def create_new_file_editor(self, path):
+        get_workbench().get_editor_notebook().open_new_file(path, remote=True)
+
     def request_dirs_child_data(self, node_id, paths):
         if get_runner():
             get_runner().send_command(
@@ -1006,9 +1202,18 @@ class BaseRemoteFileBrowser(BaseFileBrowser):
 
     def open_path_with_system_app(self, path):
         messagebox.showinfo(
-            "Not supported",
-            "Opening remote files in system app is not supported.\n\n"
-            + "Please download the file to a local directory and open it from there!",
+            tr("Not supported"),
+            tr("Opening remote files in external app is not supported.")
+            + "\n\n"
+            + tr(
+                "If it is a text file, then you can configure it to open in Thonny "
+                "by right-clicking it and selecting 'Configure ... files'."
+            )
+            + "\n\n"
+            + tr(
+                "If the file needs to be opened in external app, then download it to a local "
+                "directory and open it from there!"
+            ),
             master=self,
         )
 
@@ -1086,6 +1291,10 @@ class BaseRemoteFileBrowser(BaseFileBrowser):
 
         super().cmd_refresh_tree()
 
+    def perform_rename(self, old_name, new_name):
+        # TODO:
+        raise NotImplementedError()
+
 
 class DialogRemoteFileBrowser(BaseRemoteFileBrowser):
     def __init__(self, master, dialog):
@@ -1124,25 +1333,56 @@ class BackendFileDialog(CommonDialog):
         self.rowconfigure(0, weight=1)
 
         self.browser = DialogRemoteFileBrowser(background, self)
-        self.browser.grid(row=0, column=0, columnspan=4, sticky="nsew", pady=20, padx=20)
+        self.browser.grid(
+            row=0,
+            column=0,
+            columnspan=4,
+            sticky="nsew",
+            pady=self.get_large_padding(),
+            padx=self.get_large_padding(),
+        )
         self.browser.configure(borderwidth=1, relief="groove")
         self.browser.tree.configure(selectmode="browse")
 
         self.name_label = ttk.Label(background, text=tr("File name:"))
-        self.name_label.grid(row=1, column=0, pady=(0, 20), padx=20, sticky="w")
+        self.name_label.grid(
+            row=1,
+            column=0,
+            pady=(0, self.get_large_padding()),
+            padx=self.get_large_padding(),
+            sticky="w",
+        )
 
         self.name_var = create_string_var("")
         self.name_entry = ttk.Entry(
             background, textvariable=self.name_var, state="normal" if kind == "save" else "disabled"
         )
-        self.name_entry.grid(row=1, column=1, pady=(0, 20), padx=(0, 20), sticky="we")
+        self.name_entry.grid(
+            row=1,
+            column=1,
+            pady=(0, self.get_large_padding()),
+            padx=(0, self.get_large_padding()),
+            sticky="we",
+        )
         self.name_entry.bind("<KeyRelease>", self.on_name_edit, True)
 
         self.ok_button = ttk.Button(background, text=tr("OK"), command=self.on_ok)
-        self.ok_button.grid(row=1, column=2, pady=(0, 20), padx=(0, 20), sticky="e")
+        self.ok_button.grid(
+            row=1,
+            column=2,
+            pady=(0, self.get_large_padding()),
+            padx=(0, self.get_large_padding()),
+            sticky="e",
+        )
 
         self.cancel_button = ttk.Button(background, text=tr("Cancel"), command=self.on_cancel)
-        self.cancel_button.grid(row=1, column=3, pady=(0, 20), padx=(0, 20), sticky="e")
+        self.cancel_button.grid(
+            row=1,
+            column=3,
+            pady=(0, self.get_large_padding()),
+            padx=(0, self.get_large_padding()),
+            sticky="e",
+        )
 
         background.rowconfigure(0, weight=1)
         background.columnconfigure(1, weight=1)
@@ -1226,6 +1466,10 @@ class BackendFileDialog(CommonDialog):
         assert path.endswith(self.name_var.get())
         self.on_ok()
 
+    def set_initial_focus(self, node=None) -> bool:
+        self.name_entry.focus_set()
+        return True
+
 
 class NodeChoiceDialog(CommonDialog):
     def __init__(self, master, prompt):
