@@ -3,18 +3,18 @@
 """
 Classes used both by front-end and back-end
 """
-import ast
-import logging
 import os.path
-import platform
 import site
-import subprocess
 import sys
 from collections import namedtuple
-from typing import List, Optional, Dict, Iterable, Tuple  # @UnusedImport
+from dataclasses import dataclass
+from logging import getLogger
+from typing import Any, Callable, Dict, List, Optional, Tuple  # @UnusedImport
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
+STRING_PSEUDO_FILENAME = "<string>"
+REPL_PSEUDO_FILENAME = "<stdin>"
 MESSAGE_MARKER = "\x02"
 OBJECT_LINK_START = "[object_link_for_thonny=%d]"
 OBJECT_LINK_END = "[/object_link_for_thonny]"
@@ -54,6 +54,14 @@ FrameInfo = namedtuple(
 TextRange = namedtuple("TextRange", ["lineno", "col_offset", "end_lineno", "end_col_offset"])
 
 
+@dataclass(frozen=True)
+class DistInfo:
+    key: str
+    project_name: str
+    version: str
+    location: str
+
+
 class Record:
     def __init__(self, **kw):
         self.__dict__.update(kw)
@@ -207,6 +215,15 @@ class BackendEvent(MessageFromBackend):
         self.event_type = event_type
 
 
+class OscEvent(BackendEvent):
+    def __init__(self, text: str):
+        self.text = text
+        super().__init__("OscEvent")
+
+    def __repr__(self):
+        return f"OscEvent({self.text!r})"
+
+
 class InlineResponse(MessageFromBackend):
     def __init__(self, command_name: str, **kw) -> None:
         super().__init__(**kw)
@@ -214,10 +231,18 @@ class InlineResponse(MessageFromBackend):
         self.event_type = self.command_name + "_response"
 
 
-def serialize_message(msg: Record) -> str:
+def serialize_message(msg: Record, max_line_length=65536) -> str:
     # I want to transfer only ASCII chars because encodings are not reliable
     # (eg. can't find a way to specify PYTHONIOENCODING for cx_freeze'd program)
-    return MESSAGE_MARKER + ascii(msg)
+    # The possibility for splitting message into several lines is required because of
+    # default (safe) window size in Paramiko (https://github.com/thonny/thonny/issues/1680)
+    msg_str = ascii(msg)
+
+    lines = []
+    for i in range(0, len(msg_str), max_line_length):
+        lines.append(msg_str[i : i + max_line_length])
+
+    return MESSAGE_MARKER + str(len(lines)) + " " + "\n".join(lines)
 
 
 def parse_message(msg_string: str) -> Record:
@@ -225,7 +250,11 @@ def parse_message(msg_string: str) -> Record:
     # pylint: disable=unused-variable
     nan = float("nan")  # @UnusedVariable
     assert msg_string[0] == MESSAGE_MARKER
-    return eval(msg_string[1:])
+    assert msg_string.strip().endswith(")")
+    msg_start = msg_string.index(" ")
+    line_count = int(msg_string[1:msg_start])
+    assert line_count == msg_string.strip().count("\n") + 1
+    return eval(msg_string[msg_start:].replace("\n", ""))
 
 
 def normpath_with_actual_case(name: str) -> str:
@@ -233,35 +262,38 @@ def normpath_with_actual_case(name: str) -> str:
     if not os.path.exists(name):
         return os.path.normpath(name)
 
-    assert os.path.isabs(name) or os.path.ismount(name), "Not abs nor mount: " + name
-    assert os.path.exists(name), "Not exists: " + name
-
     if os.name == "nt":
-        # https://stackoverflow.com/questions/2113822/python-getting-filename-case-as-stored-in-windows/2114975
-        name = os.path.normpath(name)
+        try:
+            # https://stackoverflow.com/questions/2113822/python-getting-filename-case-as-stored-in-windows/2114975
+            norm_name = os.path.normpath(name)
 
-        from ctypes import create_unicode_buffer, windll
+            from ctypes import create_unicode_buffer, windll
 
-        buf = create_unicode_buffer(512)
-        # GetLongPathNameW alone doesn't fix filename part
-        windll.kernel32.GetShortPathNameW(name, buf, 512)  # @UndefinedVariable
-        windll.kernel32.GetLongPathNameW(buf.value, buf, 512)  # @UndefinedVariable
-        result = buf.value
-
-        if result.casefold() != name.casefold():
-            # Sometimes GetShortPathNameW + GetLongPathNameW doesn't work
-            # see eg. https://github.com/thonny/thonny/issues/925
-            windll.kernel32.GetLongPathNameW(name, buf, 512)  # @UndefinedVariable
+            buf = create_unicode_buffer(512)
+            # GetLongPathNameW alone doesn't fix filename part
+            windll.kernel32.GetShortPathNameW(norm_name, buf, 512)  # @UndefinedVariable
+            windll.kernel32.GetLongPathNameW(buf.value, buf, 512)  # @UndefinedVariable
             result = buf.value
 
-            if result.casefold() != name.casefold():
-                result = name
+            if result.casefold() != norm_name.casefold():
+                # Sometimes GetShortPathNameW + GetLongPathNameW doesn't work
+                # see eg. https://github.com/thonny/thonny/issues/925
+                windll.kernel32.GetLongPathNameW(norm_name, buf, 512)  # @UndefinedVariable
+                result = buf.value
 
-        if result[1] == ":":
-            # ensure drive letter is capital
-            return result[0].upper() + result[1:]
-        else:
-            return result
+                if result.casefold() != norm_name.casefold():
+                    result = norm_name
+
+            if result[1] == ":":
+                # ensure drive letter is capital
+                return result[0].upper() + result[1:]
+            else:
+                return result
+        except Exception:
+            logger.warning(
+                "Could not compute normpath_with_actual_case for %r", name, exc_info=True
+            )
+            return os.path.normpath(name)
     else:
         # easy on Linux
         # too difficult on mac
@@ -290,7 +322,7 @@ def read_source(filename):
 def get_exe_dirs():
     result = []
     if site.ENABLE_USER_SITE:
-        if platform.system() == "Windows":
+        if sys.platform == "win32":
             if site.getusersitepackages():
                 result.append(site.getusersitepackages().replace("site-packages", "Scripts"))
         else:
@@ -317,7 +349,7 @@ def get_exe_dirs():
         if os.path.isdir(dirpath) and dirpath not in result:
             result.append(dirpath)
 
-    if platform.system() != "Windows" and "/usr/local/bin" not in result:
+    if sys.platform != "win32" and "/usr/local/bin" not in result:
         # May be missing on macOS, when started as bundle
         # (yes, more may be missing, but this one is most useful)
         result.append("/usr/local/bin")
@@ -329,6 +361,8 @@ def get_site_dir(symbolic_name, executable=None):
     if not executable or executable == sys.executable:
         result = getattr(site, symbolic_name, "")
     else:
+        import subprocess
+
         result = (
             subprocess.check_output(
                 [executable, "-m", "site", "--" + symbolic_name.lower().replace("_", "-")],
@@ -345,16 +379,15 @@ def get_base_executable():
     if sys.exec_prefix == sys.base_exec_prefix:
         return sys.executable
 
-    if platform.system() == "Windows":
-        result = sys.base_exec_prefix + "\\" + os.path.basename(sys.executable)
-        result = normpath_with_actual_case(result)
-    else:
-        result = sys.executable.replace(sys.exec_prefix, sys.base_exec_prefix)
+    if sys.platform == "win32":
+        guess = sys.base_exec_prefix + "\\" + os.path.basename(sys.executable)
+        if os.path.isfile(guess):
+            return normpath_with_actual_case(guess)
 
-    if not os.path.isfile(result):
-        raise RuntimeError("Can't locate base executable")
+    if os.path.islink(sys.executable):
+        return os.path.realpath(sys.executable)
 
-    return result
+    raise RuntimeError("Don't know how to locate base executable")
 
 
 def get_augmented_system_path(extra_dirs):
@@ -370,7 +403,7 @@ def get_augmented_system_path(extra_dirs):
 def update_system_path(env, value):
     # in Windows, env keys are not case sensitive
     # this is important if env is a dict (not os.environ)
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         found = False
         for key in env:
             if key.upper() == "PATH":
@@ -383,6 +416,43 @@ def update_system_path(env, value):
         env["PATH"] = value
 
 
+@dataclass
+class SignatureParameter:
+    kind: str
+    name: str
+    annotation: Optional[str]
+    default: Optional[str]
+
+
+@dataclass
+class SignatureInfo:
+    name: str
+    params: List[SignatureParameter]
+    return_type: Optional[str]
+    current_param_index: Optional[int] = None
+    call_bracket_start: Optional[Tuple[int, int]] = None
+
+
+@dataclass
+class CompletionInfo:
+    name: str
+    name_with_symbols: str
+    full_name: str
+    type: str
+    prefix_length: int  # the number of chars to be deleted before inserting name
+    signatures: Optional[List[SignatureInfo]]
+    docstring: Optional[str]
+
+
+@dataclass
+class NameReference:
+    module_name: str
+    module_path: str
+    row: int
+    column: int
+    length: int
+
+
 class UserError(RuntimeError):
     """Errors of this class are meant to be presented without stacktrace"""
 
@@ -392,7 +462,7 @@ class UserError(RuntimeError):
 def is_hidden_or_system_file(path: str) -> bool:
     if os.path.basename(path).startswith("."):
         return True
-    elif platform.system() == "Windows":
+    elif sys.platform == "win32":
         from ctypes import windll
 
         FILE_ATTRIBUTE_HIDDEN = 0x2
@@ -413,7 +483,7 @@ def get_dirs_children_info(
 
 def get_single_dir_child_data(path: str, include_hidden: bool = False) -> Optional[Dict[str, Dict]]:
     if path == "":
-        if platform.system() == "Windows":
+        if sys.platform == "win32":
             return {**get_windows_volumes_info(), **get_windows_network_locations()}
         else:
             return get_single_dir_child_data("/", include_hidden)
@@ -548,6 +618,9 @@ def get_windows_network_locations():
     buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
     ctypes.windll.shell32.SHGetFolderPathW(0, CSIDL_NETHOOD, 0, SHGFP_TYPE_CURRENT, buf)
     shortcuts_dir = buf.value
+    if not buf.value:
+        logger.warning("Could not determine windows shortcuts directory")
+        return {}
 
     result = {}
     for entry in os.scandir(shortcuts_dir):
@@ -568,6 +641,8 @@ def get_windows_network_locations():
 
 
 def get_windows_lnk_target(lnk_file_path):
+    import subprocess
+
     import thonny
 
     script_path = os.path.join(os.path.dirname(thonny.__file__), "res", "PrintLnkTarget.vbs")
@@ -578,6 +653,8 @@ def get_windows_lnk_target(lnk_file_path):
 
 
 def execute_system_command(cmd, cwd=None, disconnect_stdin=False):
+    import subprocess
+
     logger.debug("execute_system_command, cmd=%r, cwd=%s", cmd, cwd)
     env = dict(os.environ).copy()
     encoding = "utf-8"
@@ -650,36 +727,71 @@ def get_python_version_string(version_info: Optional[Tuple] = None, maxsize=None
         result += "-" + sys.version_info[3]
 
     if maxsize is not None:
-        result += " (" + ("64" if sys.maxsize > 2 ** 32 else "32") + " bit)"
+        result += " (" + ("64" if sys.maxsize > 2**32 else "32") + " bit)"
 
     return result
 
 
-def try_load_modules_with_frontend_sys_path(module_names):
+def execute_with_frontend_sys_path(function: Callable) -> Any:
+    import ast
+
     try:
         frontend_sys_path = ast.literal_eval(os.environ["THONNY_FRONTEND_SYS_PATH"])
         assert isinstance(frontend_sys_path, list)
+        logger.info("Using THONNY_FRONTEND_SYS_PATH %s", frontend_sys_path)
     except Exception as e:
-        logger.warning("Could not get THONNY_FRONTEND_SYS_PATH", exc_info=e)
-        return
+        logger.debug("Could not get THONNY_FRONTEND_SYS_PATH", exc_info=e)
+        frontend_sys_path = []
 
-    from importlib import import_module
-
-    old_sys_path = sys.path.copy()
-    sys.path = sys.path + frontend_sys_path
+    extra_items = [item for item in frontend_sys_path if item not in sys.path]
+    sys.path = sys.path + extra_items
     try:
+        return function()
+    finally:
+        for item in extra_items:
+            if item in sys.path:
+                sys.path.remove(item)
+
+
+def try_load_modules_with_frontend_sys_path(module_names):
+    def load():
+        from importlib import import_module
+
         for name in module_names:
             try:
                 import_module(name)
             except ImportError:
                 pass
-    finally:
-        sys.path = old_sys_path
+
+    execute_with_frontend_sys_path(load)
 
 
-class ConnectionFailedException(Exception):
-    pass
+def read_one_incoming_message_str(line_reader):
+    msg_str = line_reader()
+
+    if msg_str == "":
+        return ""
+
+    if not msg_str.startswith(MESSAGE_MARKER):
+        return msg_str
+
+    line_count = int(msg_str[1:].split(maxsplit=1)[0])
+    read_lines = 1
+    while read_lines < line_count:
+        msg_str += line_reader()
+        read_lines += 1
+
+    return msg_str
 
 
-class ConnectionClosedException(Exception):
-    pass
+def is_virtual_executable(executable):
+    exe_dir = os.path.dirname(executable)
+    return os.path.exists(os.path.join(exe_dir, "activate")) or os.path.exists(
+        os.path.join(exe_dir, "activate.bat")
+    )
+
+
+def is_private_python(executable):
+    result = os.path.exists(os.path.join(os.path.dirname(executable), "thonny_python.ini"))
+    logger.debug("is_private_python(%r) == %r", executable, result)
+    return result
