@@ -1,17 +1,18 @@
 # -*- coding: utf-8 -*-
-import logging
 import os.path
+import re
 import sys
 import tkinter as tk
 import traceback
-from logging import exception
-from tkinter import messagebox, ttk
+from logging import exception, getLogger
+from tkinter import messagebox, simpledialog, ttk
+from typing import Optional
 
 from _tkinter import TclError
 
 from thonny import get_runner, get_workbench, ui_utils
 from thonny.base_file_browser import ask_backend_path, choose_node_for_file_operations
-from thonny.codeview import CodeView, BinaryFileException
+from thonny.codeview import BinaryFileException, CodeView, CodeViewText
 from thonny.common import (
     InlineCommand,
     TextRange,
@@ -23,15 +24,35 @@ from thonny.common import (
 from thonny.languages import tr
 from thonny.misc_utils import running_on_mac_os, running_on_windows
 from thonny.tktextext import rebind_control_a
-from thonny.ui_utils import askopenfilename, asksaveasfilename, select_sequence
+from thonny.ui_utils import (
+    askopenfilename,
+    asksaveasfilename,
+    get_beam_cursor,
+    select_sequence,
+    windows_known_extensions_are_hidden,
+)
 
-_dialog_filetypes = [("Python files", ".py .pyw .pyi"), ("all files", ".*")]
+PYTHON_FILES_STR = tr("Python files")
+_dialog_filetypes = [(PYTHON_FILES_STR, ".py .pyw .pyi"), (tr("all files"), ".*")]
 
 REMOTE_PATH_MARKER = " :: "
 PYTHON_EXTENSIONS = {"py", "pyw", "pyi"}
 PYTHONLIKE_EXTENSIONS = set()
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
+
+
+class EditorCodeViewText(CodeViewText):
+    """Allows separate class binding for CodeViewTexts which are inside editors"""
+
+    def __init__(self, master=None, cnf={}, **kw):
+
+        super().__init__(
+            master=master,
+            cnf=cnf,
+            **kw,
+        )
+        self.bindtags(self.bindtags() + ("EditorCodeViewText",))
 
 
 class Editor(ttk.Frame):
@@ -42,7 +63,11 @@ class Editor(ttk.Frame):
 
         # parent of codeview will be workbench so that it can be maximized
         self._code_view = CodeView(
-            get_workbench(), propose_remove_line_numbers=True, font="EditorFont"
+            get_workbench(),
+            propose_remove_line_numbers=True,
+            font="EditorFont",
+            text_class=EditorCodeViewText,
+            cursor=get_beam_cursor(),
         )
         get_workbench().event_generate(
             "EditorTextCreated", editor=self, text_widget=self.get_text_widget()
@@ -57,7 +82,6 @@ class Editor(ttk.Frame):
 
         self._filename = None
         self._last_known_mtime = None
-        self._asking_about_external_change = False
 
         self._code_view.text.bind("<<Modified>>", self._on_text_modified, True)
         self._code_view.text.bind("<<TextChange>>", self._on_text_change, True)
@@ -75,15 +99,27 @@ class Editor(ttk.Frame):
         # TODO: try to get rid of this
         return self._code_view
 
+    def get_content(self) -> str:
+        return self._code_view.get_content()
+
+    def set_filename(self, path):
+        self._filename = path
+
     def get_filename(self, try_hard=False):
         if self._filename is None and try_hard:
             self.save_file()
 
         return self._filename
 
+    def get_identifier(self):
+        if self._filename:
+            return self._filename
+        else:
+            return str(self.winfo_id())
+
     def get_title(self):
         if self.get_filename() is None:
-            result = "<untitled>"
+            result = tr("<untitled>")
         elif is_remote_path(self.get_filename()):
             path = extract_target_path(self.get_filename())
             name = path.split("/")[-1]
@@ -97,61 +133,60 @@ class Editor(ttk.Frame):
         return result
 
     def check_for_external_changes(self):
-        if self._asking_about_external_change:
-            # otherwise method will be re-entered when focus
-            # changes because of message box
-            return
-
         if self._filename is None:
             return
 
         if is_remote_path(self._filename):
             return
 
-        try:
-            self._asking_about_external_change = True
+        if self._last_known_mtime is None:
+            return
 
-            if self._last_known_mtime is None:
-                return
+        elif not os.path.exists(self._filename):
+            self.master.select(self)
 
-            elif not os.path.exists(self._filename):
+            if messagebox.askyesno(
+                tr("File is gone"),
+                tr("Looks like '%s' was deleted or moved.") % self._filename
+                + "\n\n"
+                + tr("Do you want to also close the editor?"),
+                master=self,
+            ):
+                self.master.close_editor(self)
+            else:
+                self.get_text_widget().edit_modified(True)
+                self._last_known_mtime = None
+
+        elif os.path.getmtime(self._filename) != self._last_known_mtime:
+            skip_confirmation = not self.is_modified() and get_workbench().get_option(
+                "edit.auto_refresh_saved_files"
+            )
+            if not skip_confirmation:
                 self.master.select(self)
 
-                if messagebox.askyesno(
-                    tr("File is gone"),
-                    tr("Looks like '%s' was deleted or moved outside of the editor.")
-                    % self._filename
-                    + "\n\n"
-                    + tr("Do you want to also close the editor?"),
-                    master=self,
-                ):
-                    self.master.close_editor(self)
-                else:
-                    self.get_text_widget().edit_modified(True)
-                    self._last_known_mtime = None
+            if skip_confirmation or messagebox.askyesno(
+                tr("External modification"),
+                tr("Looks like '%s' was modified outside of the editor.") % self._filename
+                + "\n\n"
+                + tr(
+                    "Do you want to discard current editor content and reload the file from disk?"
+                ),
+                master=self,
+            ):
+                prev_location = self.get_text_widget().index("insert")
+                self._load_file(self._filename, keep_undo=True)
+                try:
+                    self.get_text_widget().mark_set("insert", prev_location)
+                    self.see_line(int(prev_location.split(".")[0]))
+                except Exception:
+                    logger.exception("Could not restore previous location")
 
-            elif os.path.getmtime(self._filename) != self._last_known_mtime:
-                self.master.select(self)
-
-                if messagebox.askyesno(
-                    tr("External modification"),
-                    tr("Looks like '%s' was modified outside the editor.") % self._filename
-                    + "\n\n"
-                    + tr(
-                        "Do you want to discard current editor content and reload the file from disk?"
-                    ),
-                    master=self,
-                ):
-                    self._load_file(self._filename, keep_undo=True)
-                else:
-                    self._last_known_mtime = os.path.getmtime(self._filename)
-        finally:
-            self._asking_about_external_change = False
+            self._last_known_mtime = os.path.getmtime(self._filename)
 
     def get_long_description(self):
 
         if self._filename is None:
-            result = "<untitled>"
+            result = tr("<untitled>")
         else:
             result = self._filename
 
@@ -243,7 +278,7 @@ class Editor(ttk.Frame):
     def save_file_enabled(self):
         return self.is_modified() or not self.get_filename()
 
-    def save_file(self, ask_filename=False, save_copy=False, node=None):
+    def save_file(self, ask_filename=False, save_copy=False, node=None) -> Optional[str]:
         if self._filename is not None and not ask_filename:
             save_filename = self._filename
             get_workbench().event_generate("Save", editor=self, filename=save_filename)
@@ -285,6 +320,12 @@ class Editor(ttk.Frame):
         return save_filename
 
     def write_local_file(self, save_filename, content_bytes, save_copy):
+        process_shebang = content_bytes.startswith(b"#!/") and get_workbench().get_option(
+            "file.make_saved_shebang_scripts_executable"
+        )
+        if process_shebang:
+            content_bytes = content_bytes.replace(b"\r\n", b"\n")
+
         try:
             f = open(save_filename, mode="wb")
             f.write(content_bytes)
@@ -292,6 +333,8 @@ class Editor(ttk.Frame):
             # Force writes on disk, see https://learn.adafruit.com/adafruit-circuit-playground-express/creating-and-editing-code#1-use-an-editor-that-writes-out-the-file-completely-when-you-save-it
             os.fsync(f)
             f.close()
+            if process_shebang:
+                os.chmod(save_filename, 0o755)
             if not save_copy or save_filename == self._filename:
                 self._last_known_mtime = os.path.getmtime(save_filename)
             get_workbench().event_generate(
@@ -331,7 +374,7 @@ class Editor(ttk.Frame):
         if get_runner().ready_for_remote_file_operations(show_message=True):
             target_filename = extract_target_path(save_filename)
 
-            get_runner().send_command_and_wait(
+            result = get_runner().send_command_and_wait(
                 InlineCommand(
                     "write_file",
                     path=target_filename,
@@ -339,10 +382,17 @@ class Editor(ttk.Frame):
                     editor_id=id(self),
                     blocking=True,
                     description=tr("Saving to %s") % target_filename,
+                    make_shebang_scripts_executable=get_workbench().get_option(
+                        "file.make_saved_shebang_scripts_executable"
+                    ),
                 ),
                 dialog_title=tr("Saving"),
             )
 
+            if "error" in result:
+                messagebox.showerror(tr("Could not save"), str(result["error"]))
+                return False
+
             if not save_copy:
                 self._code_view.text.edit_modified(False)
 
@@ -355,6 +405,7 @@ class Editor(ttk.Frame):
             get_workbench().event_generate("RemoteFilesChanged")
             return True
         else:
+            messagebox.showerror(tr("Could not save"), "Back-end is not ready")
             return False
 
     def ask_new_path(self, node=None):
@@ -372,6 +423,7 @@ class Editor(ttk.Frame):
     def ask_new_remote_path(self):
         target_path = ask_backend_path(self.winfo_toplevel(), "save")
         if target_path:
+            target_path = self._check_add_py_extension(target_path)
             return make_remote_path(target_path)
         else:
             return None
@@ -384,24 +436,22 @@ class Editor(ttk.Frame):
             initialdir = os.path.dirname(self._filename)
             initialfile = os.path.basename(self._filename)
 
-        # http://tkinter.unpythonic.net/wiki/tkFileDialog
+        # https://tcl.tk/man/tcl8.6/TkCmd/getOpenFile.htm
+        type_var = tk.StringVar(value="")
         new_filename = asksaveasfilename(
             filetypes=_dialog_filetypes,
-            defaultextension=".py",
+            defaultextension=None,
             initialdir=initialdir,
             initialfile=initialfile,
             parent=get_workbench(),
+            typevariable=type_var,
         )
+        logger.info("Save dialog returned %r with typevariable %r", new_filename, type_var.get())
 
         # Different tkinter versions may return different values
         if new_filename in ["", (), None]:
             return None
 
-        # Seems that in some Python versions defaultextension
-        # acts funny
-        if new_filename.lower().endswith(".py.py"):
-            new_filename = new_filename[:-3]
-
         if running_on_windows():
             # may have /-s instead of \-s and wrong case
             new_filename = os.path.join(
@@ -409,6 +459,11 @@ class Editor(ttk.Frame):
                 os.path.basename(new_filename),
             )
 
+        if type_var.get() == PYTHON_FILES_STR or type_var.get() == "":
+            new_filename = self._check_add_py_extension(
+                new_filename, without_asking=type_var.get() == PYTHON_FILES_STR
+            )
+
         if new_filename.endswith(".py"):
             base = os.path.basename(new_filename)
             mod_name = base[:-3].lower()
@@ -450,6 +505,7 @@ class Editor(ttk.Frame):
         )
         self._code_view.text.update_tabs()
         self._code_view.text.event_generate("<<UpdateAppearance>>")
+        self._code_view.grid_main_widgets()
 
     def _listen_debugger_progress(self, event):
         # Go read-only
@@ -514,6 +570,24 @@ class Editor(ttk.Frame):
             "EditorTextDestroyed", editor=self, text_widget=self.get_text_widget()
         )
 
+    def _check_add_py_extension(self, path: str, without_asking: bool = False) -> str:
+        assert path
+        parts = re.split(r"[/\\]", path)
+        name = parts[-1]
+        if "." not in name:
+            if without_asking or messagebox.askyesno(
+                title=tr("Confirmation"),
+                message=tr("Python files usually have .py extension.")
+                + "\n\n"
+                + tr("Did you mean '%s'?" % (name + ".py")),
+                parent=self.winfo_toplevel(),
+            ):
+                return path + ".py"
+            else:
+                return path
+
+        return path
+
 
 class EditorNotebook(ui_utils.ClosableNotebook):
     """
@@ -531,6 +605,8 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().set_default("view.show_line_numbers", True)
         get_workbench().set_default("view.recommended_line_length", 0)
         get_workbench().set_default("edit.indent_with_tabs", False)
+        get_workbench().set_default("edit.auto_refresh_saved_files", True)
+        get_workbench().set_default("file.make_saved_shebang_scripts_executable", True)
 
         self._recent_menu = tk.Menu(
             get_workbench().get_menu("file"), postcommand=self._update_recent_menu
@@ -545,13 +621,23 @@ class EditorNotebook(ui_utils.ClosableNotebook):
                 self._open_file(filename)
         """
 
-        self.update_appearance()
-
         # should be in the end, so that it can be detected when
         # constructor hasn't completed yet
         self.preferred_size_in_pw = None
+        self._checking_external_changes = False
 
-        get_workbench().bind("WindowFocusIn", self.on_focus_window, True)
+        get_workbench().bind("WindowFocusIn", self.check_for_external_changes, True)
+        get_workbench().bind("ToplevelResponse", self.check_for_external_changes, True)
+        self.bind("<<NotebookTabChanged>>", self.on_tab_changed, True)
+
+    def on_tab_changed(self, *args):
+        if sys.platform == "darwin":
+            # Since Tk 8.6.11, after closing an editor, the previous editor re-appeared with
+            # widgets disappeared, at least on Aivar's machine.
+            child = self.get_current_child()
+            if child:
+                assert isinstance(child, Editor)
+                child.get_code_view().grid_main_widgets()
 
     def _init_commands(self):
         # TODO: do these commands have to be in EditorNotebook ??
@@ -644,7 +730,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             "file",
             tr("Save as..."),
             self._cmd_save_file_as,
-            default_sequence=select_sequence("<Control-Shift-S>", "<Command-Shift-s>"),
+            default_sequence=select_sequence("<Control-Shift-S>", "<Command-Shift-S>"),
             extra_sequences=["<Control-Greek_SIGMA>"],
             tester=lambda: self.get_current_editor() is not None,
             group=10,
@@ -668,6 +754,18 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             group=10,
         )
 
+        get_workbench().add_command(
+            "goto_source_line",
+            "edit",
+            tr("Go to line..."),
+            self._cmd_goto_source_line,
+            default_sequence=select_sequence("<Control-g>", "<Command-g>"),
+            # tester=,
+            # no global switch, or cross plugin switch?
+            # todo use same as find and replace -> plugins/find_replace.py
+            group=60,
+        )
+
         get_workbench().createcommand("::tk::mac::OpenDocument", self._mac_open_document)
 
     def load_startup_files(self):
@@ -751,8 +849,16 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         get_workbench().set_option("file.open_files", open_files)
 
     def _cmd_new_file(self):
+        self.open_new_file()
+
+    def open_new_file(self, path=None, remote=False):
         new_editor = Editor(self)
         get_workbench().event_generate("NewFile", editor=new_editor)
+        if path:
+            if remote:
+                new_editor.set_filename(make_remote_path(path))
+            else:
+                new_editor.set_filename(path)
         self.add(new_editor, text=new_editor.get_title())
         self.select(new_editor)
         new_editor.focus_set()
@@ -891,13 +997,20 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         if len(editors) == 1 and not editors[0].is_modified() and not editors[0].get_filename():
             self._cmd_close_file()
 
+    def _cmd_goto_source_line(self):
+        editor = self.get_current_editor()
+        if editor:
+            line_no = simpledialog.askinteger(tr("Go to line"), tr("Line number"))
+            if line_no:
+                editor.select_line(line_no)
+
     def _mac_open_document(self, *args):
         for arg in args:
             if isinstance(arg, str) and os.path.exists(arg):
                 self.show_file(arg)
         get_workbench().become_active_window()
 
-    def get_current_editor(self):
+    def get_current_editor(self) -> Optional[Editor]:
         return self.get_current_child()
 
     def get_current_editor_content(self):
@@ -905,7 +1018,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         if editor is None:
             return None
         else:
-            return editor.get_code_view().get_content()
+            return editor.get_content()
 
     def get_all_editors(self):
         # When workspace is closing, self.winfo_children()
@@ -974,7 +1087,7 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         try:
             self.indicate_modification()
         except Exception:
-            logging.exception("Could not update modification indication")
+            logger.exception("Could not update modification indication")
 
     def indicate_modification(self):
         if not running_on_mac_os():
@@ -1007,16 +1120,18 @@ class EditorNotebook(ui_utils.ClosableNotebook):
             editor.destroy()
             return None
 
-    def get_editor(self, filename, open_when_necessary=False):
-        if not is_remote_path(filename) and os.path.isfile(filename):
-            filename = normpath_with_actual_case(os.path.abspath(filename))
+    def get_editor(self, filename_or_id, open_when_necessary=False):
+        if os.path.isfile(filename_or_id):
+            filename_or_id = normpath_with_actual_case(os.path.abspath(filename_or_id))
+
         for child in self.winfo_children():
-            child_filename = child.get_filename(False)
-            if child_filename == filename:
+            assert isinstance(child, Editor)
+            child_identifier = child.get_identifier()
+            if child_identifier == filename_or_id:
                 return child
 
         if open_when_necessary:
-            return self._open_file(filename)
+            return self._open_file(filename_or_id)
         else:
             return None
 
@@ -1041,8 +1156,10 @@ class EditorNotebook(ui_utils.ClosableNotebook):
 
         if confirm:
             for editor_ in modified_editors:
+                assert isinstance(editor_, Editor)
                 if editor_.get_filename(True):
-                    editor_.save_file()
+                    if not editor_.save_file():
+                        return False
                 else:
                     return False
             return True
@@ -1052,9 +1169,18 @@ class EditorNotebook(ui_utils.ClosableNotebook):
         else:
             return True
 
-    def on_focus_window(self, event=None):
-        for editor in self.get_all_editors():
-            editor.check_for_external_changes()
+    def check_for_external_changes(self, event=None):
+        if self._checking_external_changes:
+            # otherwise the method will be re-entered when focus
+            # changes because of a confirmation message box
+            return
+
+        self._checking_external_changes = True
+        try:
+            for editor in self.get_all_editors():
+                editor.check_for_external_changes()
+        finally:
+            self._checking_external_changes = False
 
 
 def get_current_breakpoints():
