@@ -33,15 +33,15 @@ def delete_dir_try_hard(path: str, hardness: int = 5) -> None:
 
 
 def running_on_windows() -> bool:
-    return platform.system() == "Windows"
+    return sys.platform == "win32"
 
 
 def running_on_mac_os() -> bool:
-    return platform.system() == "Darwin"
+    return sys.platform == "darwin"
 
 
 def running_on_linux() -> bool:
-    return platform.system() == "Linux"
+    return sys.platform == "linux"
 
 
 def running_on_rpi() -> bool:
@@ -51,6 +51,23 @@ def running_on_rpi() -> bool:
     )
 
 
+def get_user_site_packages_dir_for_base(userbase: str) -> str:
+    # copied from site._get_path of 3.8 and 3.10
+    version = sys.version_info
+
+    if os.name == "nt":
+        if sys.version_info < (3, 10):
+            return f"{userbase}\\Python{version[0]}{version[1]}\\site-packages"
+        else:
+            ver_nodot = sys.winver.replace(".", "")
+            return f"{userbase}\\Python{ver_nodot}\\site-packages"
+
+    if sys.platform == "darwin" and sys._framework:
+        return f"{userbase}/lib/python/site-packages"
+
+    return f"{userbase}/lib/python{version[0]}.{version[1]}/site-packages"
+
+
 def list_volumes(skip_letters=set()) -> Sequence[str]:
     "Adapted from https://github.com/ntoll/uflash/blob/master/uflash.py"
     if sys.platform == "win32":
@@ -357,7 +374,7 @@ def get_file_creation_date(path_to_file):
     last modified if that isn't possible.
     See http://stackoverflow.com/a/39501288/1709587 for explanation.
     """
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         return os.path.getctime(path_to_file)
     else:
         stat = os.stat(path_to_file)
@@ -385,20 +402,6 @@ def lap_time(text=""):
     _timer_time = time.time()
 
 
-class TimeHelper:
-    def __init__(self, time_allowed):
-        self.start_time = time.time()
-        self.time_allowed = time_allowed
-
-    @property
-    def time_spent(self):
-        return time.time() - self.start_time
-
-    @property
-    def time_left(self):
-        return max(self.time_allowed - self.time_spent, 0)
-
-
 def copy_to_clipboard(data):
     if running_on_windows():
         _copy_to_windows_clipboard(data)
@@ -423,7 +426,7 @@ def copy_to_clipboard(data):
         env=env,
         universal_newlines=True,
         close_fds=True,
-        **extra
+        **extra,
     )
     proc.communicate(input=data, timeout=0.1)
 
@@ -472,26 +475,6 @@ def sizeof_fmt(num, suffix="B"):
     return "%.1f%s%s" % (num, "Yi", suffix)
 
 
-def _get_known_folder(ID):
-    # http://stackoverflow.com/a/3859336/261181
-    # http://www.installmate.com/support/im9/using/symbols/functions/csidls.htm
-    import ctypes.wintypes
-
-    SHGFP_TYPE_CURRENT = 0
-    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
-    ctypes.windll.shell32.SHGetFolderPathW(0, ID, 0, SHGFP_TYPE_CURRENT, buf)
-    assert buf.value
-    return buf.value
-
-
-def get_roaming_appdata_dir():
-    return _get_known_folder(26)
-
-
-def get_local_appdata_dir():
-    return _get_known_folder(28)
-
-
 class PopenWithOutputQueues(subprocess.Popen):
     def __init__(self, *args, **kw):
         super().__init__(*args, **kw)
@@ -522,9 +505,12 @@ def inside_flatpak():
 
 def show_command_not_available_in_flatpak_message():
     from tkinter import messagebox
+
+    from thonny import get_workbench
     from thonny.languages import tr
 
     messagebox.showinfo(
         tr("Command not available"),
         tr("This command is not available if Thonny is run via Flatpak"),
+        parent=get_workbench(),
     )
