@@ -5,50 +5,37 @@ import platform
 import sys
 import tkinter as tk
 import tkinter.font
+from logging import getLogger
 from tkinter import ttk
 
 import thonny
 from thonny import get_workbench, ui_utils
 from thonny.common import get_python_version_string
 from thonny.languages import tr
-from thonny.ui_utils import CommonDialog
+from thonny.ui_utils import CommonDialog, CommonDialogEx, create_url_label, get_hyperlink_cursor
+
+logger = getLogger(__name__)
 
 
-class AboutDialog(CommonDialog):
+class AboutDialog(CommonDialogEx):
     def __init__(self, master):
-        import webbrowser
-
         super().__init__(master)
 
-        main_frame = ttk.Frame(self)
-        main_frame.grid(sticky=tk.NSEW, ipadx=15, ipady=15)
-        main_frame.rowconfigure(0, weight=1)
-        main_frame.columnconfigure(0, weight=1)
-
         self.title(tr("About Thonny"))
         self.resizable(height=tk.FALSE, width=tk.FALSE)
-        self.protocol("WM_DELETE_WINDOW", self._ok)
 
-        # bg_frame = ttk.Frame(self) # gives proper color in aqua
-        # bg_frame.grid()
-
-        heading_font = tkinter.font.nametofont("TkHeadingFont").copy()
-        heading_font.configure(size=19, weight="bold")
+        default_heading_font = tkinter.font.nametofont("TkHeadingFont")
+        heading_font = default_heading_font.copy()
+        heading_font.configure(size=int(default_heading_font["size"] * 1.7), weight="bold")
         heading_label = ttk.Label(
-            main_frame, text="Thonny " + thonny.get_version(), font=heading_font
+            self.main_frame, text="Thonny " + thonny.get_version(), font=heading_font
         )
-        heading_label.grid()
+        heading_label.grid(pady=(self.get_large_padding(), self.get_small_padding()))
 
-        url = "https://thonny.org"
-        url_font = tkinter.font.nametofont("TkDefaultFont").copy()
-        url_font.configure(underline=1)
-        url_label = ttk.Label(
-            main_frame, text=url, style="Url.TLabel", cursor="hand2", font=url_font
-        )
+        url_label = create_url_label(self.main_frame, "https://thonny.org", justify=tk.CENTER)
         url_label.grid()
-        url_label.bind("<Button-1>", lambda _: webbrowser.open(url))
 
-        if platform.system() == "Linux":
+        if sys.platform == "linux":
             try:
                 import distro  # distro don't need to be installed
 
@@ -59,12 +46,20 @@ class AboutDialog(CommonDialog):
             if "32" not in system_desc and "64" not in system_desc:
                 system_desc += " " + self.get_os_word_size_guess()
         else:
-            system_desc = (
-                platform.system() + " " + platform.release() + " " + self.get_os_word_size_guess()
-            )
+            release = platform.release()
+            if sys.platform == "win32":
+                # Win 10 and 11 both give 10 as release
+                try:
+                    build = int(platform.version().split(".")[2])
+                    if release == "10" and build >= 22000:
+                        release = "11"
+                except Exception:
+                    logger.exception("Could not determine Windows version")
+
+            system_desc = platform.system() + " " + release + " " + self.get_os_word_size_guess()
 
         platform_label = ttk.Label(
-            main_frame,
+            self.main_frame,
             justify=tk.CENTER,
             text=system_desc
             + "\n"
@@ -74,11 +69,12 @@ class AboutDialog(CommonDialog):
             + "Tk "
             + ui_utils.get_tk_version_str(),
         )
-        platform_label.grid(pady=20)
+        platform_label.grid(pady=self.get_medium_padding())
 
-        credits_label = ttk.Label(
-            main_frame,
-            text=tr(
+        credits_label = create_url_label(
+            self.main_frame,
+            "https://github.com/thonny/thonny/blob/master/CREDITS.rst",
+            tr(
                 "Made in\n"
                 + "University of Tartu, Estonia,\n"
                 + "with the help from\n"
@@ -86,21 +82,15 @@ class AboutDialog(CommonDialog):
                 + "Raspberry Pi Foundation\n"
                 + "and Cybernetica AS"
             ),
-            style="Url.TLabel",
-            cursor="hand2",
-            font=url_font,
-            justify="center",
+            justify=tk.CENTER,
         )
         credits_label.grid()
-        credits_label.bind(
-            "<Button-1>",
-            lambda _: webbrowser.open("https://github.com/thonny/thonny/blob/master/CREDITS.rst"),
-        )
 
-        license_font = tkinter.font.nametofont("TkDefaultFont").copy()
-        license_font.configure(size=7)
+        default_font = tkinter.font.nametofont("TkDefaultFont")
+        license_font = default_font.copy()
+        license_font.configure(size=round(default_font["size"] * 0.7))
         license_label = ttk.Label(
-            main_frame,
+            self.main_frame,
             text="Copyright (©) "
             + str(datetime.datetime.now().year)
             + " Aivar Annamaa\n"
@@ -115,17 +105,15 @@ class AboutDialog(CommonDialog):
             justify=tk.CENTER,
             font=license_font,
         )
-        license_label.grid(pady=20)
+        license_label.grid(pady=self.get_medium_padding())
 
-        ok_button = ttk.Button(main_frame, text=tr("OK"), command=self._ok, default="active")
-        ok_button.grid(pady=(0, 15))
+        ok_button = ttk.Button(
+            self.main_frame, text=tr("OK"), command=self.on_close, default="active"
+        )
+        ok_button.grid(pady=(0, self.get_large_padding()))
         ok_button.focus_set()
 
-        self.bind("<Return>", self._ok, True)
-        self.bind("<Escape>", self._ok, True)
-
-    def _ok(self, event=None):
-        self.destroy()
+        self.bind("<Return>", self.on_close, True)
 
     def get_os_word_size_guess(self):
         if "32" in platform.machine() and "64" not in platform.machine():
@@ -137,7 +125,7 @@ class AboutDialog(CommonDialog):
 
 
 def load_plugin() -> None:
-    def open_about(*args):
+    def open_about():
         ui_utils.show_dialog(AboutDialog(get_workbench()))
 
     def open_url(url):
@@ -157,7 +145,7 @@ def load_plugin() -> None:
         "issues",
         "help",
         tr("Report problems"),
-        lambda: open_url("https://github.com/thonny/thonny/issues/new"),
+        lambda: open_url("https://github.com/thonny/thonny/issues"),
         group=60,
     )
     get_workbench().add_command("about", "help", tr("About Thonny"), open_about, group=61)
