@@ -1,16 +1,15 @@
 # -*- coding: utf-8 -*-
-
 import ast
-import logging
-
 import sys
 import tkinter as tk
+from logging import getLogger
+from typing import Optional
 
 from thonny import ast_utils, get_workbench, ui_utils
 from thonny.common import TextRange, range_contains_smaller
 from thonny.languages import tr
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 
 class AstView(ui_utils.TreeFrame):
@@ -26,7 +25,7 @@ class AstView(ui_utils.TreeFrame):
         self.tree.bind("<<TreeviewSelect>>", self._locate_code)
         self.tree.bind("<<Copy>>", self._copy_to_clipboard)
         self.tree.bind("<Map>", self._update, True)
-        get_workbench().get_editor_notebook().bind("<<NotebookTabChanged>>", self._update)
+        get_workbench().get_editor_notebook().bind("<<NotebookTabChanged>>", self._update, True)
         get_workbench().bind("Save", self._update, True)
         get_workbench().bind("SaveAs", self._update, True)
         get_workbench().bind_class("Text", "<Double-Button-1>", self._update, True)
@@ -42,7 +41,7 @@ class AstView(ui_utils.TreeFrame):
         self.tree.heading("range", text="Code range", anchor=tk.W)
 
         self.tree["show"] = ("headings", "tree")
-        self._current_source = None
+        self._current_source: Optional[str] = None
 
         self._update(None)
 
@@ -57,7 +56,7 @@ class AstView(ui_utils.TreeFrame):
             return
 
         new_cw = editor.get_code_view()
-        new_source = new_cw.get_content_as_bytes()
+        new_source = new_cw.get_content()
         if self._current_code_view == new_cw and self._current_source == new_source:
             return
 
