@@ -46,6 +46,7 @@ def default_light() -> SyntaxThemeSettings:
         "function_call": {},
         "method_call": {},
         "hyperlink": {"foreground": "#3A66DD", "underline": True},
+        "name_link": {"underline": True},
         # paren matcher
         "surrounding_parens": {"foreground": "Blue", "font": "BoldEditorFont"},
         "unclosed_expression": {"background": "LightGray"},
@@ -176,6 +177,7 @@ def default_dark() -> SyntaxThemeSettings:
         "function_call": {},
         "method_call": {},
         "hyperlink": {"foreground": "#619DC7", "underline": True},
+        "name_link": {"underline": True},
         # paren matcher
         "surrounding_parens": {"foreground": "#F0995B", "font": "BoldEditorFont"},
         "unclosed_expression": {"background": "#000000"},
@@ -424,4 +426,4 @@ def load_plugin() -> None:
     # Comments in IDLE Dark really hurt the eyes
     # get_workbench().add_syntax_theme("IDLE Dark", "Default Dark", idle_dark)
 
-    get_workbench().set_default("view.syntax_theme", "Default Light")
+    get_workbench().set_default("view.syntax_theme", get_workbench().get_default_syntax_theme())
