@@ -26,7 +26,6 @@ def _treeview_settings() -> BasicUiThemeSettings:
 
     return {
         "Treeview": {
-            "configure": {"font": "TreeviewFont"},
             "map": {
                 "background": [
                     ("selected", "focus", bg_sel_focus),
@@ -309,7 +308,7 @@ def clam() -> BasicUiThemeSettings:
     }
 
 
-def xpnative() -> BasicUiThemeSettings:
+def vista() -> BasicUiThemeSettings:
     # Transcribed from https://github.com/tcltk/tk/blob/master/library/ttk/xpTheme.tcl
     return {
         ".": {
@@ -419,7 +418,6 @@ def aqua() -> BasicUiThemeSettings:
         "TCombobox": {"configure": {"postoffset": [5, -2, -10, 0]}},
         "Heading": {"configure": {"font": "TkHeadingFont"}},
         "Treeview": {
-            "configure": {"rowheight": 18, "background": "white"},
             "map": {
                 "background": [
                     ("disabled", "systemDialogBackgroundInactive"),
@@ -442,7 +440,7 @@ def aqua() -> BasicUiThemeSettings:
 def windows() -> CompoundUiThemeSettings:
     tip_background = "#bbbbbb"
     return [
-        xpnative(),
+        vista(),
         _treeview_settings(),
         _menubutton_settings(),
         _paned_window_settings(),
@@ -608,8 +606,8 @@ def load_plugin() -> None:
         settings = {}  # type: Union[Dict, Callable[[], Dict]]
         if name == "clam":
             settings = clam
-        elif name == "xpnative":
-            settings = xpnative
+        elif name == "vista":
+            settings = vista
         elif name == "aqua":
             settings = aqua
 
@@ -622,8 +620,8 @@ def load_plugin() -> None:
         {"tab-close": "tab-close-clam", "tab-close-active": "tab-close-active-clam"},
     )
 
-    if "xpnative" in original_themes:
-        get_workbench().add_ui_theme("Windows", "xpnative", windows)
+    if "vista" in original_themes:
+        get_workbench().add_ui_theme("Windows", "vista", windows)
 
     if "aqua" in original_themes:
         get_workbench().add_ui_theme("Kind of Aqua", "aqua", enhanced_aqua)
