@@ -3,11 +3,11 @@
 """
 Adds debugging commands and features.
 """
-
 import ast
-import logging
 import os.path
 import tkinter as tk
+import tokenize
+from logging import getLogger
 from tkinter import ttk
 from tkinter.messagebox import showinfo
 from typing import List, Union  # @UnusedImport
@@ -30,9 +30,9 @@ from thonny.languages import tr
 from thonny.memory import VariablesFrame
 from thonny.misc_utils import running_on_mac_os, running_on_rpi, shorten_repr
 from thonny.tktextext import TextFrame
-from thonny.ui_utils import CommonDialog, select_sequence
-from thonny.ui_utils import select_sequence, CommonDialog, get_tk_version_info
-from _tkinter import TclError
+from thonny.ui_utils import CommonDialog, get_hyperlink_cursor, get_tk_version_info, select_sequence
+
+logger = getLogger(__name__)
 
 _current_debugger = None
 
@@ -50,7 +50,7 @@ class Debugger:
         self._last_debugger_command = cmd
 
         if get_runner().is_waiting_debugger_command():
-            logging.debug("_check_issue_debugger_command: %s", cmd)
+            logger.debug("_check_issue_debugger_command: %s", cmd)
 
             # tell MainCPythonBackend the state we are seeing
             cmd.setdefault(
@@ -70,7 +70,7 @@ class Debugger:
             if command == "resume":
                 self.clear_last_frame()
         else:
-            logging.debug("Bad state for sending debugger command " + str(command))
+            logger.debug("Bad state for sending debugger command " + str(command))
 
     def get_run_to_cursor_breakpoint(self):
         return None
@@ -461,7 +461,7 @@ class FrameVisualizer:
             if frame_info.current_statement is not None:
                 self._tag_range(frame_info.current_statement, stmt_tag)
             else:
-                logging.warning("Missing current_statement: %s", frame_info)
+                logger.warning("Missing current_statement: %s", frame_info)
 
         self._expression_box.update_expression(msg, frame_info)
 
@@ -647,7 +647,7 @@ class BaseExpressionBox:
                 # to while test expression
                 self.clear_debug_view()
 
-            with open(frame_info.filename, "rb") as fp:
+            with tokenize.open(frame_info.filename) as fp:
                 whole_source = fp.read()
 
             lines = whole_source.splitlines()
@@ -728,13 +728,13 @@ class BaseExpressionBox:
             lambda _: get_workbench().event_generate("ObjectSelect", object_id=value.id),
         )
 
-    def _load_expression(self, whole_source, filename, text_range):
-
+    def _load_expression(self, whole_source: str, filename, text_range):
+        assert isinstance(whole_source, str)
         root = ast_utils.parse_source(whole_source, filename)
         main_node = ast_utils.find_expression(root, text_range)
 
         source = ast_utils.extract_text_range(whole_source, text_range)
-        logging.debug("EV.load_exp: %s", (text_range, main_node, source))
+        logger.debug("EV.load_exp: %s", (text_range, main_node, source))
 
         self._clear_expression()
 
@@ -782,7 +782,7 @@ class BaseExpressionBox:
         )
 
     def _highlight_range(self, text_range, state, has_exception):
-        logging.debug("EV._highlight_range: %s", text_range)
+        logger.debug("EV._highlight_range: %s", text_range)
         self.text.tag_remove("after", "1.0", "end")
         self.text.tag_remove("before", "1.0", "end")
         self.text.tag_remove("exception", "1.0", "end")
@@ -939,6 +939,7 @@ class DialogVisualizer(CommonDialog, FrameVisualizer):
 
         self._load_code(frame_info)
         self._text_frame.text.focus()
+        self.wm_deiconify()
         self.update()
 
     def _init_layout_widgets(self, master, frame_info):
@@ -1157,7 +1158,7 @@ class ExceptionView(TextFrame):
             self.set_exception(None)
 
     def _hyperlink_enter(self, event):
-        self.text.config(cursor="hand2")
+        self.text.config(cursor=get_hyperlink_cursor())
 
     def _hyperlink_leave(self, event):
         self.text.config(cursor="")
@@ -1181,7 +1182,8 @@ def _start_debug_enabled():
     return (
         _current_debugger is None
         and get_workbench().get_editor_notebook().get_current_editor() is not None
-        and "debug" in get_runner().get_supported_features()
+        and get_runner().get_backend_proxy()
+        and get_runner().get_backend_proxy().can_debug()
     )
 
 
