@@ -1,4 +1,3 @@
-import logging
 import os
 import signal
 import subprocess
@@ -6,50 +5,37 @@ import threading
 import time
 import tkinter as tk
 from collections import OrderedDict
+from logging import getLogger
 from tkinter import messagebox, ttk
 from typing import Optional
 
 from thonny import get_runner, ui_utils
 from thonny.languages import tr
-from thonny.misc_utils import (
-    running_on_windows,
-)
+from thonny.misc_utils import running_on_windows
 from thonny.plugins.micropython import (
     BareMetalMicroPythonConfigPage,
     BareMetalMicroPythonProxy,
     add_micropython_backend,
     list_serial_ports_with_descriptions,
 )
-from thonny.running import get_interpreter_for_subprocess
+from thonny.plugins.micropython.mp_front import get_uart_adapter_vids_pids
+from thonny.running import get_front_interpreter_for_subprocess
 from thonny.workdlg import WorkDialog
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
+
+VIDS_PIDS_TO_AVOID_IN_ESP_BACKENDS = set()
 
 
 class ESPProxy(BareMetalMicroPythonProxy):
-    @classmethod
-    def _is_potential_port(cls, p):
-        # They have UART adapter
-        return (
-            (p.vid, p.pid) in cls.get_known_usb_vids_pids()
-            or (p.vid, None) in cls.get_known_usb_vids_pids()
-            or p.description in cls.get_known_port_descriptions()
-            or cls.should_consider_unknown_devices()
-            and (p.vid, p.pid) not in cls.get_vids_pids_to_avoid()
-            and (
-                ("USB" in p.description and "serial" in p.description.lower())
-                or "UART" in p.description
-                or "DAPLink" in p.description
-                or "STLink" in p.description
-            )
-            and getattr(p, "manufacturer", "") != "MicroPython"  # adapter can't have this?
-            and "python" not in p.description.lower()
-        )
-
     @classmethod
     def get_vids_pids_to_avoid(self):
-        # micro:bit
-        return {(0x0D28, 0x0204)}
+        return VIDS_PIDS_TO_AVOID_IN_ESP_BACKENDS
+
+    def _get_backend_launcher_path(self) -> str:
+        import thonny.plugins.esp.esp_back
+
+        return thonny.plugins.esp.esp_back.__file__
 
 
 class ESP8266Proxy(ESPProxy):
@@ -58,19 +44,13 @@ class ESP8266Proxy(ESPProxy):
 
     @classmethod
     def get_known_usb_vids_pids(cls):
-        return cls.get_uart_adapter_vids_pids()
-
-    def _get_api_stubs_path(self):
-        return os.path.join(os.path.dirname(__file__), "esp8266_api_stubs")
+        return get_uart_adapter_vids_pids()
 
 
 class ESP32Proxy(ESPProxy):
     @classmethod
     def get_known_usb_vids_pids(cls):
-        return cls.get_uart_adapter_vids_pids()
-
-    def _get_api_stubs_path(self):
-        return os.path.join(os.path.dirname(__file__), "esp32_api_stubs")
+        return get_uart_adapter_vids_pids()
 
 
 class ESPConfigPage(BareMetalMicroPythonConfigPage):
@@ -102,7 +82,7 @@ class ESPConfigPage(BareMetalMicroPythonConfigPage):
         try:
             import esptool
 
-            return [get_interpreter_for_subprocess(), "-u", "-m", "esptool"]
+            return [get_front_interpreter_for_subprocess(), "-u", "-m", "esptool"]
         except ImportError:
             import shutil
 
@@ -160,8 +140,8 @@ class ESPFlashingDialog(WorkDialog):
         return 35
 
     def populate_main_frame(self):
-        epadx = self.get_padding()
-        ipadx = self.get_internal_padding()
+        epadx = self.get_large_padding()
+        ipadx = self.get_small_padding()
         epady = epadx
         ipady = ipadx
 
@@ -183,12 +163,14 @@ class ESPFlashingDialog(WorkDialog):
         firmware_label = ttk.Label(self.main_frame, text="Firmware")
         firmware_label.grid(row=2, column=1, sticky="w", padx=(epadx, 0), pady=(ipady, 0))
 
-        self._firmware_entry = ttk.Entry(self.main_frame, width=55)
+        self._firmware_entry = ttk.Entry(self.main_frame, width=50)
         self._firmware_entry.grid(row=2, column=2, sticky="nsew", padx=ipadx, pady=(ipady, 0))
 
         browse_button = ttk.Button(self.main_frame, text="Browse...", command=self._browse)
         browse_button.grid(row=2, column=3, sticky="we", padx=(0, epadx), pady=(ipady, 0))
 
+        self.main_frame.columnconfigure(2, weight=1)
+
         # FLASH_MODE
         self._flashmode = tk.StringVar(None, "keep")
         flashmode_group = ttk.Labelframe(self.main_frame, text="Flash mode")
