@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 
 import tkinter as tk
+from logging import getLogger
 from tkinter import ttk
 
 from thonny import get_workbench
 from thonny.languages import tr
-from thonny.ui_utils import CommonDialog, select_sequence
+from thonny.ui_utils import CommonDialog, select_sequence, show_dialog
 
 # TODO - consider moving the cmd_find method to main class in order to pass the editornotebook reference
 # TODO - logging
@@ -17,6 +18,8 @@ from thonny.ui_utils import CommonDialog, select_sequence
 
 _active_find_dialog = None
 
+logger = getLogger(__name__)
+
 
 class FindDialog(CommonDialog):
 
@@ -64,7 +67,17 @@ class FindDialog(CommonDialog):
         self.find_label.grid(column=0, row=0, sticky="w", padx=(padx, 0), pady=(pady, 0))
 
         # Find text field
-        self.find_entry_var = tk.StringVar()
+        initial_text = ""
+        try:
+            widget = get_workbench().focus_get()
+            if isinstance(widget, tk.Text):
+                selected_lines = widget.selection_get().splitlines()
+                if selected_lines:
+                    initial_text = selected_lines[0]
+        except Exception:
+            logger.exception("Could not get initial text")
+
+        self.find_entry_var = tk.StringVar(value=initial_text)
         self.find_entry = ttk.Entry(main_frame, textvariable=self.find_entry_var)
         self.find_entry.grid(column=1, row=0, columnspan=2, padx=(0, 10), pady=(pady, 0))
         if FindDialog.last_searched_word is not None:
@@ -164,6 +177,10 @@ class FindDialog(CommonDialog):
         self.find_entry.focus_set()
         self.find_entry.selection_range(0, tk.END)
 
+    def _raise_tags(self):
+        self.codeview.text.tag_raise("found")
+        self.codeview.text.tag_raise("current_found")
+
     # callback for text modifications on the find entry object, used to dynamically enable and disable buttons
     def _update_button_statuses(self, *args):
         find_text = self.find_entry_var.get()
@@ -291,6 +308,7 @@ class FindDialog(CommonDialog):
                 self.codeview.text.tag_add(
                     "found", self.active_found_tag[0], self.active_found_tag[1]
                 )
+                self._raise_tags()
 
         else:  # start a new search, start from the current insert line position
             if self.active_found_tag is not None:
@@ -334,6 +352,7 @@ class FindDialog(CommonDialog):
         self.codeview.text.tag_add(
             "current_found", wordstart, wordend
         )  # tags the found word as active
+        self._raise_tags()
         self.active_found_tag = (wordstart, wordend)
         self.replace_and_find_button.config(state="normal")
         self.replace_button.config(state="normal")
@@ -391,18 +410,20 @@ class FindDialog(CommonDialog):
             endpos = self.codeview.text.index("%s+%dc" % (currentpos, len(tofind)))
             self.passive_found_tags.add((currentpos, endpos))
             self.codeview.text.tag_add("found", currentpos, endpos)
+            self._raise_tags()
 
             currentpos = self.codeview.text.index("%s+1c" % currentpos)
 
 
 def load_plugin() -> None:
-    def cmd_open_find_dialog():
+    def cmd_open_find_dialog(event=None):
         if _active_find_dialog is not None:
             _active_find_dialog.focus_set()
         else:
             editor = get_workbench().get_editor_notebook().get_current_editor()
             if editor:
-                FindDialog(editor._code_view)
+                dlg = FindDialog(editor._code_view)
+                show_dialog(dlg)
 
     def find_f3(event):
         if _active_find_dialog is None:
@@ -419,4 +440,6 @@ def load_plugin() -> None:
         extra_sequences=["<Control-Greek_phi>"],
     )
 
+    get_workbench().bind("<<CtrlFInText>>", cmd_open_find_dialog, True)
+
     get_workbench().bind("<F3>", find_f3, True)
