@@ -1,20 +1,24 @@
-import logging
 import pathlib
-import platform
 import sys
 import threading
 import time
+from logging import getLogger
 from textwrap import dedent
 
-from thonny.plugins.micropython.bare_metal_backend import NORMAL_PROMPT, FIRST_RAW_PROMPT
-from thonny.common import ConnectionFailedException
-from thonny.plugins.micropython.connection import MicroPythonConnection
+from .connection import MicroPythonConnection
 
-logger = logging.getLogger(__name__)
+OUTPUT_ENQ = b"\x05"
+OUTPUT_ACK = b"\x06"
+
+NORMAL_PROMPT = b">>> "
+FIRST_RAW_PROMPT = b"raw REPL; CTRL-B to exit\r\n>"
+
+
+logger = getLogger(__name__)
 
 
 class SerialConnection(MicroPythonConnection):
-    def __init__(self, port, baudrate, dtr=None, rts=None, skip_reader=False):
+    def __init__(self, port, baudrate=115200, dtr=None, rts=None, skip_reader=False):
 
         import serial
         from serial.serialutil import SerialException
@@ -22,7 +26,9 @@ class SerialConnection(MicroPythonConnection):
         super().__init__()
 
         try:
-            self._serial = serial.Serial(port=None, baudrate=baudrate, timeout=None, exclusive=True)
+            self._serial = serial.Serial(
+                port=None, baudrate=baudrate, timeout=None, write_timeout=2, exclusive=True
+            )
             # Tweaking dtr and rts was proposed by
             # https://github.com/thonny/thonny/pull/1187
             # but in some cases it messes up communication.
@@ -45,7 +51,7 @@ class SerialConnection(MicroPythonConnection):
             message = "Unable to connect to " + port + ": " + err_str
 
             # TODO: check if these error codes also apply to Linux and Mac
-            if error.errno == 13 and platform.system() == "Linux":
+            if error.errno == 13 and sys.platform == "linux":
                 try:
                     group = pathlib.Path(self._serial.port).group()
                 except Exception as e:
@@ -71,7 +77,7 @@ class SerialConnection(MicroPythonConnection):
             elif error.errno == 16:
                 message += "\n\n" + "Try restarting the device."
 
-            raise ConnectionFailedException(message)
+            raise ConnectionRefusedError(message)
 
         if skip_reader:
             self._reading_thread = None
@@ -79,7 +85,7 @@ class SerialConnection(MicroPythonConnection):
             self._reading_thread = threading.Thread(target=self._listen_serial, daemon=True)
             self._reading_thread.start()
 
-    def write(self, data):
+    def write(self, data: bytes) -> int:
         size = self._serial.write(data)
         # print(data.decode(), end="")
         assert size == len(data)
@@ -96,10 +102,20 @@ class SerialConnection(MicroPythonConnection):
                     # print("LISTEN EOFFFFFFFFFF")
                     break
                 data += self._serial.read_all()
+                # logger.debug("GOT %r", data)
+
+                if data.endswith(OUTPUT_ENQ) and self.text_mode:
+                    # Flow control.
+                    logger.debug("Read ENQ, responding with ACK")
+                    # Assuming connection is idle and it is safe to write in this thread
+                    self._serial.write(OUTPUT_ACK)
+                    self._serial.flush()
+                    data = data[:-1]
+                    continue
 
                 # don't publish incomplete utf-8 data
                 try:
-                    if self.unicode_guard:
+                    if self.text_mode:
                         data.decode("utf-8")  # testing if data decodes
                     to_be_published = data
                     data = b""
@@ -125,9 +141,6 @@ class SerialConnection(MicroPythonConnection):
     def outgoing_is_empty(self):
         return self._serial.out_waiting == 0
 
-    def reset_output_buffer(self):
-        self._serial.reset_output_buffer()
-
     def close(self):
         if self._serial is not None:
             try:
@@ -139,7 +152,7 @@ class SerialConnection(MicroPythonConnection):
                     self._serial.close()
                     self._serial = None
                 except Exception:
-                    logging.exception("Couldn't close serial")
+                    logger.exception("Couldn't close serial")
 
 
 class DifficultSerialConnection(SerialConnection):
@@ -163,7 +176,3 @@ class DifficultSerialConnection(SerialConnection):
             super()._make_output_available(data[end - 1 :], block=block)
         else:
             super()._make_output_available(data, block=block)
-
-
-def debug(*args, file=sys.stderr):
-    print(*args, file=file)
