@@ -1,12 +1,11 @@
 import sys
 import threading
-import time
+from logging import DEBUG, getLogger
 from queue import Queue
 
-from thonny.common import ConnectionFailedException
-from thonny.plugins.micropython.connection import MicroPythonConnection
+from .connection import MicroPythonConnection
 
-DEBUG = False
+logger = getLogger(__name__)
 
 
 class WebReplConnection(MicroPythonConnection):
@@ -29,7 +28,7 @@ class WebReplConnection(MicroPythonConnection):
                 "Can't import `websockets`. You can install it via 'Tools => Manage plug-ins'.",
                 file=sys.stderr,
             )
-            sys.exit(-1)
+            sys.exit(1)
         self._url = url
         self._password = password
         self._write_responses = Queue()
@@ -50,7 +49,8 @@ class WebReplConnection(MicroPythonConnection):
         import asyncio
 
         loop = asyncio.new_event_loop()
-        loop.set_debug(DEBUG)
+        if logger.isEnabledFor(DEBUG):
+            loop.set_debug(True)
         loop.run_until_complete(self._ws_main())
 
     async def _ws_main(self):
@@ -66,7 +66,7 @@ class WebReplConnection(MicroPythonConnection):
         await asyncio.gather(self._ws_keep_reading(), self._ws_keep_writing())
 
     async def _ws_connect(self):
-        import websockets
+        import websockets.exceptions
 
         try:
             try:
@@ -76,20 +76,18 @@ class WebReplConnection(MicroPythonConnection):
                 self._ws = await websockets.connect(self._url, ping_interval=None)
         except OSError as e:
             # print("\nCould not connect:", e, file=sys.stderr)
-            raise ConnectionFailedException(str(e))
-        debug("GOT WS", self._ws)
+            raise ConnectionRefusedError(str(e))
+        logger.debug("GOT WS: %r", self._ws)
 
         # read password prompt and send password
         read_chars = ""
+        logger.debug("Looking for password prompt")
         while read_chars != "Password: ":
-            debug("prelude", read_chars)
             ch = await self._ws.recv()
-            debug("GOT", ch)
             read_chars += ch
 
-        debug("sending password")
+        logger.debug("Submitting password")
         await self._ws.send(self._password + "\n")
-        debug("sent password")
 
     async def _ws_keep_reading(self):
         import websockets.exceptions
@@ -116,18 +114,18 @@ class WebReplConnection(MicroPythonConnection):
         while True:
             while not self._write_queue.empty():
                 data = self._write_queue.get(block=False)
-                if isinstance(data, WebreplBinaryMsg):
-                    payload = data.data
-                else:
+                if self.text_mode:
                     payload = data.decode("UTF-8")
+                else:
+                    payload = data
                 await self._ws.send(payload)
-                debug("Wrote bytes", len(data))
+                # logger.debug("Wrote %r bytes", len(data))
                 self._write_responses.put(len(data))
 
             # Allow reading loop to progress
             await asyncio.sleep(0.01)
 
-    def write(self, data):
+    def write(self, data: bytes) -> int:
         self._write_queue.put_nowait(data)
         return self._write_responses.get()
 
@@ -144,19 +142,3 @@ class WebReplConnection(MicroPythonConnection):
         import asyncio
         asyncio.get_event_loop().run_until_complete(self.async_close())
         """
-
-
-class WebreplBinaryMsg:
-    """This wrapper helps distinguishing between bytes which should
-    be decoded and sent as text frame and bytes sent as binary frame"""
-
-    def __init__(self, data):
-        self.data = data
-
-    def __len__(self):
-        return len(self.data)
-
-
-def debug(*args):
-    if DEBUG:
-        print(*args, file=sys.stderr)
