@@ -1,7 +1,8 @@
 import ast
 import logging
 import tkinter as tk
-from tkinter import ttk, messagebox
+from logging import getLogger
+from tkinter import messagebox, ttk
 
 import thonny.memory
 from thonny import get_runner, get_workbench, ui_utils
@@ -12,6 +13,8 @@ from thonny.misc_utils import shorten_repr
 from thonny.tktextext import TextFrame
 from thonny.ui_utils import ems_to_pixels
 
+logger = logging.getLogger(__name__)
+
 
 class ObjectInspector(ttk.Frame):
     def __init__(self, master):
@@ -134,6 +137,7 @@ class ObjectInspector(ttk.Frame):
             [
                 FileHandleInspector(self.content_page),
                 FunctionInspector(self.content_page),
+                MicrobitImageInspector(self.content_page),
                 StringInspector(self.content_page),
                 ElementsInspector(self.content_page),
                 DictInspector(self.content_page),
@@ -267,12 +271,12 @@ class ObjectInspector(ttk.Frame):
         if self.back_links == []:
             self.back_label.config(foreground="lightgray", cursor="arrow")
         else:
-            self.back_label.config(foreground="blue", cursor="hand2")
+            self.back_label.config(foreground="blue", cursor=get_hyperlink_cursor())
 
         if self.forward_links == []:
             self.forward_label.config(foreground="lightgray", cursor="arrow")
         else:
-            self.forward_label.config(foreground="blue", cursor="hand2")
+            self.forward_label.config(foreground="blue", cursor=get_hyperlink_cursor())
         """
 
     def update_type_specific_info(self, object_info):
@@ -325,7 +329,7 @@ class FileHandleInspector(TextFrame, ContentInspector):
     def set_object_info(self, object_info):
 
         if "file_content" not in object_info:
-            logging.exception("File error: " + object_info["file_error"])
+            logger.exception("File error: " + object_info["file_error"])
             return
 
         assert "file_content" in object_info
@@ -414,6 +418,20 @@ class StringInspector(TextFrame, ContentInspector):
         """
 
 
+class MicrobitImageInspector(TextFrame, ContentInspector):
+    def __init__(self, master):
+        ContentInspector.__init__(self, master)
+        TextFrame.__init__(self, master, read_only=True)
+
+    def applies_to(self, object_info):
+        return "microbit_image" in object_info
+
+    def set_object_info(self, object_info):
+        content = object_info["microbit_image"]
+        # self.text.configure(height=min(line_count_sep, 10))
+        self.text.set_content(content)
+
+
 class IntInspector(TextFrame, ContentInspector):
     def __init__(self, master):
         ContentInspector.__init__(self, master)
