@@ -1,7 +1,7 @@
 import os.path
 from tkinter.messagebox import askyesno, showinfo
-from thonny import get_workbench, THONNY_USER_DIR
-import shutil
+
+from thonny import THONNY_USER_DIR, get_workbench
 
 
 def load_plugin():
@@ -9,8 +9,12 @@ def load_plugin():
     if os.path.exists(old_data_dir):
 
         def doit():
+            import shutil
+
             if not os.path.exists(old_data_dir):
-                showinfo("Already deleted", "Looks like it's already deleted")
+                showinfo(
+                    "Already deleted", "Looks like it's already deleted", master=get_workbench()
+                )
                 return
 
             answer = askyesno(
@@ -22,11 +26,12 @@ def load_plugin():
                 + "If you don't intend to use older Thonny versions anymore, "
                 + "you probably want to delete the old directory and reclaim disk space.\n\n"
                 + "Do you want me to delete this directory now?",
+                master=get_workbench(),
             )
             if answer:
                 shutil.rmtree(old_data_dir, True)
                 shutil.rmtree(old_data_dir, True)  # first one may keep empty directory
-                showinfo("Done!", "Done!")
+                showinfo("Done!", "Done!", master=get_workbench())
 
         get_workbench().add_command(
             "delolddatadir", "tools", "Clean up Thonny 2.1 data folder ...", doit, group=110
