@@ -5,7 +5,7 @@ import warnings
 
 
 def _clear_screen():
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         os.system("cls")
     else:
         os.system("clear")
@@ -40,13 +40,13 @@ def is_virtual_exe(p):
     )
 
 
-def is_bundled_exe(p):
+def is_private_exe(p):
     pdir = os.path.dirname(p)
     return os.path.exists(os.path.join(pdir, "thonny_python.ini"))
 
 
 def list_commands(prefix, highlighted_reals, highlighted_dirs):
-    for suffix in ["", "3", "3.5", "3.6", "3.7", "3.8"]:
+    for suffix in ["", "3", "3.8", "3.9", "3.10", "3.11"]:
         cmd = prefix + suffix
         import shutil
 
@@ -80,10 +80,10 @@ def wrap_in_ansi_code(text, code):
 
 
 def can_use_ansi_codes():
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         ver = platform.win32_ver()
         try:
-            return int(ver[0]) >= 10
+            return float(ver[0]) >= 10
         except Exception:
             warnings.warn("Can't determine Windows version %s" % (ver,))
             return False
@@ -139,12 +139,12 @@ if __name__ == "__main__":
     sys_real = normpath_with_actual_case(equivalent_realpath(sys.executable))
     sys_executable = normpath_with_actual_case(sys.executable)
 
-    if is_virtual_exe(sys_executable) or is_bundled_exe(sys_executable):
+    if is_virtual_exe(sys_executable) or is_private_exe(sys_executable):
         highlight_dirs = [os.path.dirname(sys_executable)]
     else:
         highlight_dirs = []
 
-    if platform.system() == "Windows":
+    if sys.platform == "win32":
         # Add Scripts for pip
         highlight_dirs.append(os.path.join(os.path.dirname(sys_real), "Scripts"))
         highlight_dirs.append(os.path.join(os.path.dirname(sys_executable), "Scripts"))
