@@ -1,12 +1,13 @@
-import logging
 import tkinter as tk
 import traceback
+from logging import getLogger
 
 from thonny import get_workbench, ui_utils
 from thonny.codeview import get_syntax_options_for_tag
 from thonny.tktextext import TweakableText
+from thonny.ui_utils import get_hyperlink_cursor
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 
 class RstText(TweakableText):
@@ -20,7 +21,7 @@ class RstText(TweakableText):
                 "font": "TkDefaultFont",
                 # "cursor" : "",
                 **kw,
-            }
+            },
         )
         self.configure_tags()
         self._visitor = None
@@ -467,7 +468,7 @@ class RstText(TweakableText):
         return self._visitor
 
     def _hyperlink_enter(self, event):
-        self.config(cursor="hand2")
+        self.config(cursor=get_hyperlink_cursor())
 
     def _hyperlink_leave(self, event):
         self.config(cursor="")
