@@ -7,29 +7,25 @@ Commands get executed via shell, this way the command line in the
 shell becomes kind of title for the execution.
 
 """
-
-
 import collections
-import logging
 import os.path
 import re
 import shlex
-import signal
 import subprocess
-import traceback
-
 import sys
 import time
 import tkinter as tk
+import traceback
 import warnings
-from logging import debug
+from abc import ABC, abstractmethod
+from logging import getLogger
 from threading import Thread
 from time import sleep
 from tkinter import messagebox, ttk
-from typing import Any, List, Optional, Set, Union, Callable  # @UnusedImport; @UnusedImport
+from typing import Any, Callable, Dict, List, Optional, Set, Union  # @UnusedImport; @UnusedImport
 
 import thonny
-from thonny import THONNY_USER_DIR, common, get_runner, get_shell, get_workbench
+from thonny import THONNY_USER_DIR, common, get_runner, get_shell, get_workbench, report_time
 from thonny.common import (
     BackendEvent,
     CommandToBackend,
@@ -37,26 +33,27 @@ from thonny.common import (
     DebuggerResponse,
     EOFCommand,
     InlineCommand,
+    InlineResponse,
     InputSubmission,
+    MessageFromBackend,
     ToplevelCommand,
     ToplevelResponse,
     UserError,
     is_same_path,
-    normpath_with_actual_case,
     parse_message,
     path_startswith,
+    read_one_incoming_message_str,
     serialize_message,
-    update_system_path,
-    MessageFromBackend,
     universal_relpath,
+    update_system_path,
 )
 from thonny.editors import (
+    extract_target_path,
     get_current_breakpoints,
     get_saved_current_script_filename,
-    is_remote_path,
-    is_local_path,
     get_target_dirname_from_editor_filename,
-    extract_target_path,
+    is_local_path,
+    is_remote_path,
 )
 from thonny.languages import tr
 from thonny.misc_utils import (
@@ -66,10 +63,10 @@ from thonny.misc_utils import (
     running_on_windows,
     show_command_not_available_in_flatpak_message,
 )
-from thonny.ui_utils import CommonDialogEx, select_sequence, show_dialog
+from thonny.ui_utils import select_sequence, show_dialog
 from thonny.workdlg import WorkDialog
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 WINDOWS_EXE = "python.exe"
 OUTPUT_MERGE_THRESHOLD = 1000
@@ -78,11 +75,10 @@ RUN_COMMAND_LABEL = ""  # init later when gettext is ready
 RUN_COMMAND_CAPTION = ""
 EDITOR_CONTENT_TOKEN = "$EDITOR_CONTENT"
 
-EXPECTED_TERMINATION_CODE = 123
-
 INTERRUPT_SEQUENCE = "<Control-c>"
 
-ANSI_CODE_TERMINATOR = re.compile("[@-~]")
+CSI_TERMINATOR = re.compile("[@-~]")
+OSC_TERMINATOR = re.compile(r"\a|\x1B\\")
 
 # other components may turn it on in order to avoid grouping output lines into one event
 io_animation_required = False
@@ -96,19 +92,11 @@ class Runner:
 
         self._init_commands()
         self._state = "starting"
-        self._proxy = None  # type: BackendProxy
+        self._proxy: Optional[BackendProxy] = None
         self._publishing_events = False
         self._polling_after_id = None
         self._postponed_commands = []  # type: List[CommandToBackend]
-
-    def _remove_obsolete_jedi_copies(self) -> None:
-        # Thonny 2.1 used to copy jedi in order to make it available
-        # for the backend. Get rid of it now
-        for item in os.listdir(THONNY_USER_DIR):
-            if item.startswith("jedi_0."):
-                import shutil
-
-                shutil.rmtree(os.path.join(THONNY_USER_DIR, item), True)
+        self._last_accepted_backend_command = None
 
     def start(self) -> None:
         global _console_allocated
@@ -120,8 +108,6 @@ class Runner:
             _console_allocated = False
 
         self.restart_backend(False, True)
-        # temporary
-        self._remove_obsolete_jedi_copies()
 
     def _init_commands(self) -> None:
         global RUN_COMMAND_CAPTION, RUN_COMMAND_LABEL
@@ -218,7 +204,7 @@ class Runner:
 
     def _set_state(self, state: str) -> None:
         if self._state != state:
-            logging.debug("Runner state changed: %s ==> %s" % (self._state, state))
+            logger.debug("Runner state changed: %s ==> %s" % (self._state, state))
             self._state = state
 
     def is_running(self):
@@ -255,9 +241,7 @@ class Runner:
             and not self.is_waiting_debugger_command()
         ):
             get_workbench().bell()
-            logging.warning(
-                "RUNNER: Command %s was attempted at state %s" % (cmd, self.get_state())
-            )
+            logger.warning("RUNNER: Command %s was attempted at state %s" % (cmd, self.get_state()))
             return
 
         # Attach extra info
@@ -269,8 +253,12 @@ class Runner:
 
         cmd["local_cwd"] = get_workbench().get_local_cwd()
 
+        if self._proxy.running_inline_command and isinstance(cmd, InlineCommand):
+            self._postpone_command(cmd)
+            return
+
         # Offer the command
-        logging.debug("RUNNER Sending: %s, %s", cmd.name, cmd)
+        logger.debug("RUNNER Sending: %s, %s", cmd.name, cmd)
         response = self._proxy.send_command(cmd)
 
         if response == "discard":
@@ -281,6 +269,9 @@ class Runner:
         else:
             assert response is None
             get_workbench().event_generate("CommandAccepted", command=cmd)
+            self._last_accepted_backend_command = cmd
+            if isinstance(cmd, InlineCommand):
+                self._proxy.running_inline_command = True
 
         if isinstance(cmd, (ToplevelCommand, DebuggerCommand)):
             self._set_state("running")
@@ -289,7 +280,7 @@ class Runner:
             # This may be only logical restart, which does not look like restart to the runner
             get_workbench().event_generate("BackendRestart", full=False)
 
-    def send_command_and_wait(self, cmd: CommandToBackend, dialog_title: str) -> MessageFromBackend:
+    def send_command_and_wait(self, cmd: InlineCommand, dialog_title: str) -> MessageFromBackend:
         dlg = InlineCommandDialog(get_workbench(), cmd, title=dialog_title + " ...")
         show_dialog(dlg)
         return dlg.response
@@ -302,7 +293,7 @@ class Runner:
                     self._postponed_commands.remove(older_cmd)
 
         if len(self._postponed_commands) > 10:
-            logging.warning("Can't pile up too many commands. This command will be just ignored")
+            logger.warning("Can't pile up too many commands. This command will be just ignored")
         else:
             self._postponed_commands.append(cmd)
 
@@ -311,7 +302,7 @@ class Runner:
         self._postponed_commands = []
 
         for cmd in todo:
-            logging.debug("Sending postponed command: %s", cmd)
+            # logger.debug("Sending postponed command: %s", cmd) # too much spam
             self.send_command(cmd)
 
     def send_program_input(self, data: str) -> None:
@@ -342,6 +333,14 @@ class Runner:
         get_shell().submit_magic_command(cd_cmd_line + exe_cmd_line)
 
     def execute_editor_content(self, command_name, args):
+        if command_name.lower() == "debug":
+            messagebox.showinfo(
+                tr("Information"),
+                tr("For debugging the program must be saved first."),
+                master=get_workbench(),
+            )
+            return
+
         get_shell().submit_magic_command(
             construct_cmd_line(
                 ["%" + command_name, "-c", EDITOR_CONTENT_TOKEN] + args, [EDITOR_CONTENT_TOKEN]
@@ -354,13 +353,56 @@ class Runner:
         current file/script and submit it to shell
         """
 
+        if not self._proxy:
+            return
+
+        assert (
+            command_name[0].isupper()
+            or command_name == "run"
+            and not self._proxy.should_restart_interpreter_before_run()
+        )
+
         if not self.is_waiting_toplevel_command():
-            self.restart_backend(True, False, 2)
+            self._proxy.interrupt()
 
-        filename = get_saved_current_script_filename()
+            try:
+                self._wait_for_prompt(1)
+            except TimeoutError:
+                # turtle.mainloop, for example, is not easy to interrupt.
+                get_shell().print_error("Could not interrupt current process. ")
+                wait_instructions = "Please wait, try again or select Stop/Restart!"
 
-        if not filename:
-            # user has cancelled file saving
+                if self._proxy.stop_restart_kills_user_program():
+                    get_shell().print_error("Forcing the program to stop.\n")
+                    self.cmd_stop_restart()
+                    try:
+                        self._wait_for_prompt(1)
+                    except TimeoutError:
+                        get_shell().print_error(
+                            "Could not force-stop the program. " + wait_instructions + "\n"
+                        )
+                else:
+                    # For some back-ends (e.g. BareMetalMicroPython) killing is not an option.
+                    # Besides, they may be configured to avoid refreshing the environment
+                    # before run (for performance reasons).
+                    get_shell().print_error(wait_instructions + "\n")
+                    return
+
+        editor = get_workbench().get_editor_notebook().get_current_editor()
+        if not editor:
+            return
+
+        UNTITLED = "<untitled>"
+        if editor.get_filename():
+            filename = editor.save_file()
+            if not filename:
+                # user has cancelled file saving
+                return
+        else:
+            filename = UNTITLED
+
+        if not self._proxy:
+            # Saving the file may have killed the proxy
             return
 
         if (
@@ -368,6 +410,7 @@ class Runner:
             and not self._proxy.can_run_remote_files()
             or is_local_path(filename)
             and not self._proxy.can_run_local_files()
+            or filename == UNTITLED
         ):
             self.execute_editor_content(command_name, self._get_active_arguments())
         else:
@@ -384,6 +427,16 @@ class Runner:
                 target_path, self._get_active_arguments(), working_directory, command_name
             )
 
+    def _wait_for_prompt(self, timeout) -> None:
+        start_time = time.time()
+        while time.time() - start_time <= timeout:
+            if self.is_waiting_toplevel_command():
+                return
+            get_workbench().update()
+            sleep(0.01)
+
+        raise TimeoutError("Backend did not respond in time")
+
     def _get_active_arguments(self):
         if get_workbench().get_option("view.show_program_arguments"):
             args_str = get_workbench().get_option("run.program_arguments")
@@ -394,21 +447,25 @@ class Runner:
 
     def cmd_run_current_script_enabled(self) -> bool:
         return (
-            get_workbench().get_editor_notebook().get_current_editor() is not None
-            and "run" in get_runner().get_supported_features()
+            self._proxy and get_workbench().get_editor_notebook().get_current_editor() is not None
         )
 
     def _cmd_run_current_script_in_terminal_enabled(self) -> bool:
         return (
             self._proxy
-            and "run_in_terminal" in self._proxy.get_supported_features()
+            and self._proxy.can_run_in_terminal()
             and self.cmd_run_current_script_enabled()
         )
 
     def cmd_run_current_script(self) -> None:
         if get_workbench().in_simple_mode():
             get_workbench().hide_view("VariablesView")
-        self.execute_current("Run")
+        report_time("Before Run")
+        if self._proxy and self._proxy.should_restart_interpreter_before_run():
+            self.execute_current("Run")
+        else:
+            self.execute_current("run")
+        report_time("After Run")
 
     def _cmd_run_current_script_in_terminal(self) -> None:
         if inside_flatpak():
@@ -437,7 +494,7 @@ class Runner:
                     master=self,
                 )
         else:
-            logging.warning("User tried interrupting without proxy")
+            logger.warning("User tried interrupting without proxy")
 
     def _cmd_interrupt_with_shortcut(self, event=None):
         if not self._cmd_interrupt_enabled():
@@ -478,7 +535,7 @@ class Runner:
         if get_workbench().in_simple_mode():
             get_workbench().hide_view("VariablesView")
 
-        self.restart_backend(True)
+        self.restart_backend(clean=True)
 
     def disconnect(self):
         proxy = self.get_backend_proxy()
@@ -498,7 +555,7 @@ class Runner:
                 "Can't perform this action",
                 "Ctrl+D only has effect on an empty line / prompt.\n"
                 + "Submit current input (press ENTER) and try again",
-                master=self,
+                master=get_workbench(),
             )
             return
 
@@ -531,18 +588,13 @@ class Runner:
                 msg = self._proxy.fetch_next_message()
                 if not msg:
                     break
-                logging.debug(
+                logger.debug(
                     "RUNNER GOT: %s, %s in state: %s", msg.event_type, msg, self.get_state()
                 )
 
                 msg_count += 1
             except BackendTerminatedError as exc:
-                self._report_backend_crash(exc)
-                self.destroy_backend()
-                return False
-
-            if msg.get("SystemExit", False):
-                self.restart_backend(True)
+                self._handle_backend_termination(exc.returncode)
                 return False
 
             # change state
@@ -550,6 +602,9 @@ class Runner:
                 self._set_state("waiting_toplevel_command")
             elif isinstance(msg, DebuggerResponse):
                 self._set_state("waiting_debugger_command")
+            elif isinstance(msg, InlineResponse):
+                # next inline command won't be sent before response from the last has arrived
+                self._proxy.running_inline_command = False
             else:
                 "other messages don't affect the state"
 
@@ -571,39 +626,38 @@ class Runner:
 
         self._send_postponed_commands()
 
-    def _report_backend_crash(self, exc: Exception) -> None:
-        returncode = getattr(exc, "returncode", "?")
-        err = "Backend terminated or disconnected."
+    def _handle_backend_termination(self, returncode: int) -> None:
+        err = f"Process ended with exit code {returncode}."
 
         try:
             faults_file = os.path.join(THONNY_USER_DIR, "backend_faults.log")
             if os.path.exists(faults_file):
-                with open(faults_file, encoding="ASCII") as fp:
+                with open(faults_file, encoding="ASCII", errors="replace") as fp:
                     err += fp.read()
         except Exception:
-            logging.exception("Failed retrieving backend faults")
+            logger.exception("Failed retrieving backend faults")
 
-        err = err.strip() + " Use 'Stop/Restart' to restart.\n"
-
-        if returncode != EXPECTED_TERMINATION_CODE:
-            get_workbench().event_generate("ProgramOutput", stream_name="stderr", data="\n" + err)
+        get_workbench().event_generate("ProgramOutput", stream_name="stderr", data="\n" + err)
 
         get_workbench().become_active_window(False)
+        self.destroy_backend()
+        if self._should_restart_after_command(self._last_accepted_backend_command):
+            self.restart_backend(clean=True, automatic=True)
 
-    def restart_backend(self, clean: bool, first: bool = False, wait: float = 0) -> None:
+    def _should_restart_after_command(self, cmd: CommandToBackend):
+        if cmd is None or not hasattr(cmd, "name"):
+            return False
+
+        return cmd.name in ["Run", "run", "Debug", "debug", "execute_source"]
+
+    def restart_backend(self, clean: bool, first: bool = False, automatic: bool = False) -> None:
         """Recreate (or replace) backend proxy / backend process."""
 
-        if not first:
-            get_shell().restart()
-            get_shell().update_idletasks()
-
         self.destroy_backend()
         backend_name = get_workbench().get_option("run.backend_name")
         if backend_name not in get_workbench().get_backends():
             raise UserError(
-                "Can't find backend '{}'. Please select another backend from options".format(
-                    backend_name
-                )
+                f"Unknown interpreter kind '{backend_name}'. Please select another interpreter from options"
             )
 
         backend_class = get_workbench().get_backends()[backend_name].proxy_class
@@ -613,33 +667,22 @@ class Runner:
 
         self._poll_backend_messages()
 
-        if wait:
-            start_time = time.time()
-            while not self.is_waiting_toplevel_command() and time.time() - start_time <= wait:
-                # self._pull_backend_messages()
-                # TODO: update in a loop can be slow on Mac https://core.tcl-lang.org/tk/tktview/f642d7c0f4
-                get_workbench().update()
-                sleep(0.01)
+        if not first:
+            get_shell().restart(automatic=automatic)
+            get_shell().update_idletasks()
 
         get_workbench().event_generate("BackendRestart", full=True)
 
-    def destroy_backend(self) -> None:
+    def destroy_backend(self, for_restart: bool = False) -> None:
         if self._polling_after_id is not None:
             get_workbench().after_cancel(self._polling_after_id)
             self._polling_after_id = None
 
         self._postponed_commands = []
         if self._proxy:
-            self._proxy.destroy()
+            self._proxy.destroy(for_restart=for_restart)
             self._proxy = None
-
-        get_workbench().event_generate("BackendTerminated")
-
-    def get_local_executable(self) -> Optional[str]:
-        if self._proxy is None:
-            return None
-        else:
-            return self._proxy.get_local_executable()
+            get_workbench().event_generate("BackendTerminated")
 
     def get_backend_proxy(self) -> "BackendProxy":
         return self._proxy
@@ -669,7 +712,7 @@ class Runner:
             result = kernel32.AttachConsole(child.pid)
             if not result:
                 err = ctypes.get_last_error()
-                logging.info("Could not allocate console. Error code: " + str(err))
+                logger.info("Could not allocate console. Error code: " + str(err))
             child.stdin.write(b"\n")
             try:
                 child.stdin.flush()
@@ -697,12 +740,6 @@ class Runner:
 
         return ready
 
-    def get_supported_features(self) -> Set[str]:
-        if self._proxy is None:
-            return set()
-        else:
-            return self._proxy.get_supported_features()
-
     def supports_remote_files(self):
         if self._proxy is None:
             return False
@@ -722,12 +759,12 @@ class Runner:
             return self._proxy.get_node_label()
 
     def using_venv(self) -> bool:
-        from thonny.plugins.cpython import CPythonProxy
+        from thonny.plugins.cpython_frontend import LocalCPythonProxy
 
-        return isinstance(self._proxy, CPythonProxy) and self._proxy._in_venv
+        return isinstance(self._proxy, LocalCPythonProxy) and self._proxy._in_venv
 
 
-class BackendProxy:
+class BackendProxy(ABC):
     """Communicates with backend process.
 
     All communication methods must be non-blocking,
@@ -744,25 +781,24 @@ class BackendProxy:
         Backend is considered ready when the runner gets a ToplevelResponse
         with attribute "welcome_text" from fetch_next_message.
         """
+        self.running_inline_command = False
 
+    @abstractmethod
     def send_command(self, cmd: CommandToBackend) -> Optional[str]:
         """Send the command to backend. Return None, 'discard' or 'postpone'"""
-        raise NotImplementedError()
 
+    @abstractmethod
     def send_program_input(self, data: str) -> None:
         """Send input data to backend"""
-        raise NotImplementedError()
 
+    @abstractmethod
     def fetch_next_message(self):
         """Read next message from the queue or None if queue is empty"""
-        raise NotImplementedError()
-
-    def run_script_in_terminal(self, script_path, args, interactive, keep_open):
-        raise NotImplementedError()
 
+    @abstractmethod
     def get_sys_path(self):
         "backend's sys.path"
-        return []
+        ...
 
     def get_backend_name(self):
         return type(self).backend_name
@@ -770,32 +806,36 @@ class BackendProxy:
     def get_pip_gui_class(self):
         return None
 
+    @abstractmethod
     def interrupt(self):
         """Tries to interrupt current command without resetting the backend"""
-        pass
+        ...
 
-    def destroy(self):
+    @abstractmethod
+    def destroy(self, for_restart: bool = False):
         """Called when Thonny no longer needs this instance
         (Thonny gets closed or new backend gets selected)
         """
-        pass
+        ...
 
+    @abstractmethod
     def is_connected(self):
-        return True
+        ...
 
-    def get_local_executable(self):
-        """Return system command for invoking current interpreter"""
-        return None
+    @abstractmethod
+    def has_local_interpreter(self):
+        ...
 
-    def get_supported_features(self):
-        return {"run"}
+    @abstractmethod
+    def get_target_executable(self) -> Optional[str]:
+        """Returns `sys.executable` if the interpreter has it."""
 
     def get_node_label(self):
-        """Used as files caption if back-end has separate files"""
+        """Used as files caption if back-end has separate files,
+        also when choosing file save target"""
         return "Back-end"
 
     def get_full_label(self):
-        """Used in pip GUI title"""
         return self.get_node_label()
 
     def supports_remote_files(self):
@@ -806,6 +846,12 @@ class BackendProxy:
         """Whether it runs code from local files"""
         return True
 
+    def should_restart_interpreter_before_run(self) -> bool:
+        return True
+
+    def stop_restart_kills_user_program(self) -> bool:
+        return True
+
     def supports_remote_directories(self):
         return False
 
@@ -818,69 +864,113 @@ class BackendProxy:
     def can_run_local_files(self):
         raise NotImplementedError()
 
+    @abstractmethod
+    def can_debug(self) -> bool:
+        ...
+
     def ready_for_remote_file_operations(self):
         return False
 
-    def get_cwd(self):
+    def get_cwd(self) -> Optional[str]:
         return None
 
-    def get_clean_description(self):
-        return self.backend_description
-
-    @classmethod
-    def get_current_switcher_configuration(cls):
+    @abstractmethod
+    def get_current_switcher_configuration(self) -> Dict[str, Any]:
         """returns the dict of configuration entries that distinguish current backend conf from other
-        items in the backend switcher"""
-        return {"run.backend_name": cls.backend_name}
+        items in the backend switcher. Also used for collecting last used configurations."""
 
     @classmethod
+    @abstractmethod
+    def get_switcher_configuration_label(cls, conf: Dict[str, Any]) -> str:
+        """Formats configuration for menu item"""
+
+    @classmethod
+    @abstractmethod
     def get_switcher_entries(cls):
         """
         Each returned entry creates one item in the backend switcher menu.
         """
-        return [(cls.get_current_switcher_configuration(), cls.backend_description)]
+
+    @abstractmethod
+    def can_run_in_terminal(self) -> bool:
+        ...
+
+    def run_script_in_terminal(self, script_path, args, interactive, keep_open):
+        raise NotImplementedError()
 
     def has_custom_system_shell(self):
         return False
 
-    def open_custom_system_shell(self):
+    def open_custom_system_shell(self) -> None:
         raise NotImplementedError()
 
 
-class SubprocessProxy(BackendProxy):
-    def __init__(self, clean: bool, executable: Optional[str] = None) -> None:
+class SubprocessProxy(BackendProxy, ABC):
+    def __init__(self, clean: bool, mgmt_executable: Optional[str] = None) -> None:
         super().__init__(clean)
 
-        if executable:
-            self._executable = executable
+        if mgmt_executable:
+            self._mgmt_executable = mgmt_executable
         else:
-            self._executable = get_interpreter_for_subprocess()
+            self._mgmt_executable = get_front_interpreter_for_subprocess()
 
-        if ".." in self._executable:
-            self._executable = os.path.normpath(self._executable)
+        if ".." in self._mgmt_executable:
+            self._mgmt_executable = os.path.normpath(self._mgmt_executable)
 
-        if not os.path.isfile(self._executable):
-            raise UserError(
-                "Interpreter '%s' does not exist. Please check the configuration!"
-                % self._executable
-            )
         self._welcome_text = ""
 
         self._proc = None
-        self._terminated_readers = 0
         self._response_queue = None
         self._sys_path = []
         self._usersitepackages = None
+        self._reported_executable = None
         self._gui_update_loop_id = None
         self._in_venv = None
         self._cwd = self._get_initial_cwd()  # pylint: disable=assignment-from-none
         self._start_background_process(clean=clean)
+        self._have_check_remembered_current_configuration = False
+
+    def _check_remember_current_configuration(self) -> None:
+        current_configuration = self.get_current_switcher_configuration()
+        if not self._should_remember_configuration(current_configuration):
+            return
+
+        last_configurations = self.get_last_configurations()
+        if current_configuration in last_configurations:
+            last_configurations.remove(current_configuration)
+
+        last_configurations.insert(0, current_configuration)
+
+        # remove non-valid
+        last_configurations = [
+            conf for conf in last_configurations if self.is_valid_configuration(conf)
+        ]
+        # Remember only last 5
+        last_configurations = last_configurations[:5]
+        self.set_last_configurations(last_configurations)
+
+    def _should_remember_configuration(self, configuration: Dict[str, Any]) -> bool:
+        return True
+
+    @classmethod
+    def get_last_configurations(cls) -> List[Dict]:
+        return get_workbench().get_option(f"{cls.backend_name}.last_configurations")
+
+    @classmethod
+    def set_last_configurations(cls, value: List[Dict[str, Any]]) -> None:
+        return get_workbench().set_option(f"{cls.backend_name}.last_configurations", value)
+
+    @classmethod
+    @abstractmethod
+    def is_valid_configuration(cls, conf: Dict[str, Any]) -> bool:
+        """Returns whether it makes sense to present this configuration in the switcher"""
+        ...
 
     def _get_initial_cwd(self):
         return None
 
     def _get_environment(self):
-        env = get_environment_for_python_subprocess(self._executable)
+        env = get_environment_for_python_subprocess(self._mgmt_executable)
         # variables controlling communication with the back-end process
         env["PYTHONIOENCODING"] = "utf-8"
 
@@ -904,15 +994,19 @@ class SubprocessProxy(BackendProxy):
         # deque, because in one occasion I need to put messages back
         self._response_queue = collections.deque()
 
-        if not os.path.exists(self._executable):
-            raise UserError(
-                "Interpreter (%s) not found. Please recheck corresponding option!"
-                % self._executable
+        if not os.path.exists(self._mgmt_executable):
+            get_shell().print_error(
+                f"Interpreter {self._mgmt_executable!r} not found.\nPlease select another!"
             )
+            return
+            # raise UserError(
+            #    "Interpreter (%s) not found. Please recheck corresponding option!"
+            #    % self._mgmt_executable
+            # )
 
         cmd_line = (
             [
-                self._executable,
+                self._mgmt_executable,
                 "-u",  # unbuffered IO
                 "-B",  # don't write pyo/pyc files
                 # (to avoid problems when using different Python versions without write permissions)
@@ -925,14 +1019,11 @@ class SubprocessProxy(BackendProxy):
         if running_on_windows():
             creationflags = subprocess.CREATE_NEW_PROCESS_GROUP
 
-        debug("Starting the backend: %s %s", cmd_line, get_workbench().get_local_cwd())
-
-        extra_params = {}
-        if sys.version_info >= (3, 6):
-            extra_params["encoding"] = "utf-8"
+        logger.info("Starting the backend: %s %s", cmd_line, get_workbench().get_local_cwd())
 
         self._proc = subprocess.Popen(
             cmd_line,
+            executable=cmd_line[0],
             bufsize=0,
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
@@ -941,14 +1032,18 @@ class SubprocessProxy(BackendProxy):
             env=self._get_environment(),
             universal_newlines=True,
             creationflags=creationflags,
-            **extra_params
+            encoding="utf-8",
         )
 
+        self._send_initial_input()
+
         # setup asynchronous output listeners
-        self._terminated_readers = 0
         Thread(target=self._listen_stdout, args=(self._proc.stdout,), daemon=True).start()
         Thread(target=self._listen_stderr, args=(self._proc.stderr,), daemon=True).start()
 
+    def _send_initial_input(self) -> None:
+        pass
+
     def _get_launch_cwd(self):
         return self.get_cwd() if self.uses_local_filesystem() else None
 
@@ -958,7 +1053,8 @@ class SubprocessProxy(BackendProxy):
     def send_command(self, cmd: CommandToBackend) -> Optional[str]:
         """Send the command to backend. Return None, 'discard' or 'postpone'"""
         if isinstance(cmd, ToplevelCommand) and cmd.name[0].isupper():
-            self._clear_environment()
+            self._prepare_clean_launch()
+            logger.info("Prepared clean state for executing %r", cmd)
 
         if isinstance(cmd, ToplevelCommand):
             # required by SshCPythonBackend for creating fresh target process
@@ -972,10 +1068,14 @@ class SubprocessProxy(BackendProxy):
             self._send_msg(cmd)
 
     def _send_msg(self, msg):
+        if not self._proc:
+            logger.warning("Ignoring command without active backend process")
+            return
+
         self._proc.stdin.write(serialize_message(msg) + "\n")
         self._proc.stdin.flush()
 
-    def _clear_environment(self):
+    def _prepare_clean_launch(self):
         pass
 
     def send_program_input(self, data):
@@ -993,18 +1093,18 @@ class SubprocessProxy(BackendProxy):
     def get_sys_path(self):
         return self._sys_path
 
-    def destroy(self):
+    def destroy(self, for_restart: bool = False):
         self._close_backend()
 
     def _close_backend(self):
         if self._proc is not None and self._proc.poll() is None:
+            logger.info("Killing backend process")
             self._proc.kill()
 
         self._proc = None
         self._response_queue = None
 
     def _listen_stdout(self, stdout):
-        # debug("... started listening to stdout")
         # will be called from separate thread
 
         # allow self._response_queue to be replaced while processing
@@ -1024,13 +1124,15 @@ class SubprocessProxy(BackendProxy):
 
         while True:
             try:
-                data = stdout.readline()
+                data = read_one_incoming_message_str(stdout.readline)
             except IOError:
+                # TODO: What's this ???
                 sleep(0.1)
                 continue
 
             # debug("... read some stdout data", repr(data))
             if data == "":
+                logger.info("Reader got EOF")
                 break
             else:
                 try:
@@ -1060,12 +1162,9 @@ class SubprocessProxy(BackendProxy):
                                 )
                             )
 
-        self._terminated_readers += 1
-
     def _listen_stderr(self, stderr):
-        # stderr is used only for debugger debugging
         while True:
-            data = stderr.readline()
+            data = read_one_incoming_message_str(stderr.readline)
             if data == "":
                 break
             else:
@@ -1073,8 +1172,6 @@ class SubprocessProxy(BackendProxy):
                     BackendEvent("ProgramOutput", stream_name="stderr", data=data)
                 )
 
-        self._terminated_readers += 1
-
     def _store_state_info(self, msg):
         if "cwd" in msg:
             self._cwd = msg["cwd"]
@@ -1105,9 +1202,6 @@ class SubprocessProxy(BackendProxy):
         if self.uses_local_filesystem():
             get_workbench().set_local_cwd(cwd)
 
-    def get_supported_features(self):
-        return {"run"}
-
     def get_site_packages(self):
         # NB! site.sitepackages may not be present in virtualenv
         for d in self._sys_path:
@@ -1129,13 +1223,18 @@ class SubprocessProxy(BackendProxy):
 
     def fetch_next_message(self):
         if not self._response_queue or len(self._response_queue) == 0:
-            if self.is_terminated() and self._terminated_readers == 2:
+            if self.is_terminated():
                 raise BackendTerminatedError(self._proc.returncode if self._proc else None)
             else:
                 return None
 
         msg = self._response_queue.popleft()
-        self._store_state_info(msg)
+        if isinstance(msg, ToplevelResponse):
+            self._store_state_info(msg)
+            if not self._have_check_remembered_current_configuration:
+                self._check_remember_current_configuration()
+                self._have_check_remembered_current_configuration = True
+
         if msg.event_type == "ProgramOutput":
             # combine available small output messages to one single message,
             # in order to put less pressure on UI code
@@ -1173,50 +1272,40 @@ class SubprocessProxy(BackendProxy):
 
 
 def _ends_with_incomplete_ansi_code(data):
-    pos = data.rfind("\033")
+    pos = max(data.rfind("\033["), data.rfind("\033]"))
+
     if pos == -1:
         return False
 
-    # note ANSI_CODE_TERMINATOR also includes [
+    if len(data) < 3:
+        return True
+
+    # note CSI_TERMINATOR also includes [
     params_and_terminator = data[pos + 2 :]
-    return not ANSI_CODE_TERMINATOR.search(params_and_terminator)
-
-
-def is_bundled_python(executable):
-    return os.path.exists(os.path.join(os.path.dirname(executable), "thonny_python.ini"))
-
-
-def create_backend_python_process(
-    args, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
-):
-    """Used for running helper commands (eg. pip) on CPython backend.
-    Assumes current backend is CPython."""
-
-    # TODO: if backend == frontend, then delegate to create_frontend_python_process
-
-    python_exe = get_runner().get_local_executable()
-
-    env = get_environment_for_python_subprocess(python_exe)
-    env["PYTHONIOENCODING"] = "utf-8"
-    env["PYTHONUNBUFFERED"] = "1"
-
-    # TODO: remove frontend python from path and add backend python to it
-
-    return _create_python_process(python_exe, args, stdin, stdout, stderr, env=env)
+    if data[1] == "[":
+        return not CSI_TERMINATOR.search(params_and_terminator)
+    elif data[1] == "]":
+        return not OSC_TERMINATOR.search(params_and_terminator)
 
 
 def create_frontend_python_process(
-    args, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
+    args,
+    stdin=None,
+    stdout=subprocess.PIPE,
+    stderr=subprocess.STDOUT,
+    environment_extras: Optional[Dict[str, str]] = None,
 ):
     """Used for running helper commands (eg. for installing plug-ins on by the plug-ins)"""
     if _console_allocated:
-        python_exe = get_interpreter_for_subprocess().replace("pythonw.exe", "python.exe")
+        python_exe = get_front_interpreter_for_subprocess().replace("pythonw.exe", "python.exe")
     else:
-        python_exe = get_interpreter_for_subprocess().replace("python.exe", "pythonw.exe")
+        python_exe = get_front_interpreter_for_subprocess().replace("python.exe", "pythonw.exe")
     env = get_environment_for_python_subprocess(python_exe)
     env["PYTHONIOENCODING"] = "utf-8"
     env["PYTHONUNBUFFERED"] = "1"
-    return _create_python_process(python_exe, args, stdin, stdout, stderr)
+    if environment_extras is not None:
+        env.update(environment_extras)
+    return _create_python_process(python_exe, args, stdin, stdout, stderr, env=env)
 
 
 def _create_python_process(
@@ -1281,7 +1370,7 @@ def is_venv_interpreter_of_current_interpreter(executable):
     return False
 
 
-def get_environment_for_python_subprocess(target_executable):
+def get_environment_for_python_subprocess(target_executable) -> Dict[str, str]:
     overrides = get_environment_overrides_for_python_subprocess(target_executable)
     return get_environment_with_overrides(overrides)
 
@@ -1369,7 +1458,7 @@ def get_environment_overrides_for_python_subprocess(target_executable):
                 result["TCL_LIBRARY"] = get_workbench().tk.exprstring("$tcl_library")
                 result["TK_LIBRARY"] = get_workbench().tk.exprstring("$tk_library")
         except Exception:
-            logging.exception("Can't compute Tcl/Tk library location")
+            logger.exception("Can't compute Tcl/Tk library location")
 
     return result
 
@@ -1408,6 +1497,7 @@ class InlineCommandDialog(WorkDialog):
         get_workbench().bind("InlineResponse", self._on_response, True)
         get_workbench().bind("InlineProgress", self._on_progress, True)
         get_workbench().bind("ProgramOutput", self._on_output, True)
+        get_workbench().bind("BackendTerminated", self._on_backend_terminated, True)
 
         super().__init__(master, autostart=autostart)
 
@@ -1443,6 +1533,11 @@ class InlineCommandDialog(WorkDialog):
 
             self.report_done(success)
 
+    def _on_backend_terminated(self, msg):
+        self.set_action_text("Error!")
+        self.response = dict(error="Backend terminated")
+        self.report_done(False)
+
     def _on_progress(self, msg):
         if msg.get("command_id") != getattr(self._cmd, "id"):
             return
@@ -1490,10 +1585,10 @@ class InlineCommandDialog(WorkDialog):
 def get_frontend_python():
     # TODO: deprecated (name can be misleading)
     warnings.warn("get_frontend_python is deprecated")
-    return get_interpreter_for_subprocess(sys.executable)
+    return get_front_interpreter_for_subprocess(sys.executable)
 
 
-def get_interpreter_for_subprocess(candidate=None):
+def get_front_interpreter_for_subprocess(candidate=None):
     if candidate is None:
         candidate = sys.executable
 
