@@ -1,15 +1,16 @@
 # coding=utf-8
 """Extensions for tk.Text"""
-import logging
-import platform
+import sys
 import time
+import tkinter
 import tkinter as tk
-from logging import exception
+from logging import getLogger
 from tkinter import TclError
 from tkinter import font as tkfont
 from tkinter import ttk
+from typing import Optional
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 
 class TweakableText(tk.Text):
@@ -20,6 +21,8 @@ class TweakableText(tk.Text):
 
         self._read_only = read_only
         self._suppress_events = False
+        self._edit_count: int = 0
+        self._last_operation_time: float = time.time()
 
         self._original_widget_name = self._w + "_orig"
         self.tk.call("rename", self._w, self._original_widget_name)
@@ -63,19 +66,38 @@ class TweakableText(tk.Text):
             ):
                 pass
             else:
-                exception(
+                logger.exception(
                     "[_dispatch_tk_operation] operation: " + operation + ", args:" + repr(args)
                 )
                 # traceback.print_exc()
 
             return ""  # Taken from idlelib.WidgetRedirector
 
+        except Exception as e:
+            # Need to catch to avoid the crash?
+            logger.exception("Exception in _dispatch_tk_operation")
+            from tkinter import messagebox
+
+            messagebox.showerror(
+                "Internal error",
+                "Error in _dispatch_tk_operation\n" + str(e),
+                parent=tkinter._default_root,
+            )
+
     def set_read_only(self, value):
         self._read_only = value
 
     def is_read_only(self):
         return self._read_only
 
+    def get_edit_count(self) -> int:
+        return self._edit_count
+
+    def get_last_operation_time(self) -> float:
+        """Returns the timestamp of the widget creation or time or the last time
+        the text or cursor location was changed"""
+        return self._last_operation_time
+
     def set_content(self, chars):
         self.direct_delete("1.0", tk.END)
         self.direct_insert("1.0", chars)
@@ -130,7 +152,7 @@ class TweakableText(tk.Text):
 
     def direct_mark(self, *args):
         self._original_mark(*args)
-
+        self._last_operation_time = time.time()
         if args[:2] == ("set", "insert") and not self._suppress_events:
             self.event_generate("<<CursorMove>>")
 
@@ -160,11 +182,15 @@ class TweakableText(tk.Text):
 
     def direct_insert(self, index, chars, tags=None, **kw):
         self._original_insert(index, chars, tags, **kw)
+        self._edit_count += 1
+        self._last_operation_time = time.time()
         if not self._suppress_events:
             self.event_generate("<<TextChange>>")
 
     def direct_delete(self, index1, index2=None, **kw):
         self._original_delete(index1, index2, **kw)
+        self._edit_count += 1
+        self._last_operation_time = time.time()
         if not self._suppress_events:
             self.event_generate("<<TextChange>>")
 
@@ -236,6 +262,8 @@ class EnhancedText(TweakableText):
         self.bind("<Control-Delete>", if_not_readonly(self.delete_word_right), True)
         self.bind("<Control-d>", self._redirect_ctrld, True)
         self.bind("<Control-t>", self._redirect_ctrlt, True)
+        self.bind("<Control-f>", self._redirect_ctrlf, True)
+        self.bind("<Shift-Control-space>", self._redirect_shift_control_space, True)
         self.bind("<BackSpace>", if_not_readonly(self.perform_smart_backspace), True)
         self.bind("<Return>", if_not_readonly(self.perform_return), True)
         self.bind("<KP_Enter>", if_not_readonly(self.perform_return), True)
@@ -246,7 +274,7 @@ class EnhancedText(TweakableText):
         except Exception:
             pass
 
-        if platform.system() == "Windows":
+        if sys.platform == "win32":
             self.bind("<KeyPress>", self._insert_untypable_characters_on_windows, True)
 
     def _bind_keypad(self):
@@ -311,12 +339,24 @@ class EnhancedText(TweakableText):
         self.event_generate("<<CtrlDInText>>")
         return "break"
 
+    def _redirect_shift_control_space(self, event):
+        # I want to disable the effect of Shift-Control-space in the text but still
+        # keep the event for other purposes
+        self.event_generate("<<ShiftControlSpaceInText>>")
+        return "break"
+
     def _redirect_ctrlt(self, event):
         # I want to disable the swap effect of Ctrl-T in the text but still
         # keep the event for other purposes
         self.event_generate("<<CtrlTInText>>")
         return "break"
 
+    def _redirect_ctrlf(self, event):
+        # I want to disable the Tk effect of Ctrl-T in the text but still
+        # keep the event for other purposes
+        self.event_generate("<<CtrlFInText>>")
+        return "break"
+
     def tag_reset(self, tag_name):
         empty_conf = {key: "" for key in self.tag_configure(tag_name)}
         self.tag_configure(empty_conf)
@@ -763,7 +803,7 @@ class TextFrame(ttk.Frame):
         horizontal_scrollbar_style=None,
         borderwidth=0,
         relief="sunken",
-        **text_options
+        **text_options,
     ):
         ttk.Frame.__init__(self, master=master, borderwidth=borderwidth, relief=relief)
 
@@ -779,25 +819,27 @@ class TextFrame(ttk.Frame):
         }
         final_text_options.update(text_options)
         self.text = text_class(self, **final_text_options)
-        self.text.grid(row=0, column=2, sticky=tk.NSEW)
 
         if vertical_scrollbar:
             self._vbar = vertical_scrollbar_class(
                 self, orient=tk.VERTICAL, style=vertical_scrollbar_style
             )
-            self._vbar.grid(row=0, column=3, sticky=tk.NSEW)
             self._vbar["command"] = self._vertical_scroll
             self.text["yscrollcommand"] = self._vertical_scrollbar_update
+        else:
+            self._vbar = None
 
         if horizontal_scrollbar:
             self._hbar = horizontal_scrollbar_class(
                 self, orient=tk.HORIZONTAL, style=horizontal_scrollbar_style
             )
-            self._hbar.grid(row=1, column=0, sticky=tk.NSEW, columnspan=3)
             self._hbar["command"] = self._horizontal_scroll
             self.text["xscrollcommand"] = self._horizontal_scrollbar_update
+        else:
+            self._hbar = None
 
-        self.columnconfigure(2, weight=1)
+        self.grid_main_widgets()
+        self.columnconfigure(1, weight=1)
         self.rowconfigure(0, weight=1)
 
         self._ui_theme_change_binding = self.bind(
@@ -808,6 +850,13 @@ class TextFrame(ttk.Frame):
 
         self._reload_theme_options(None)
 
+    def grid_main_widgets(self):
+        self.text.grid(row=0, column=1, sticky=tk.NSEW)
+        if self._vbar:
+            self._vbar.grid(row=0, column=2, sticky=tk.NSEW)
+        if self._hbar:
+            self._hbar.grid(row=1, column=0, sticky=tk.NSEW, columnspan=3)
+
     def focus_set(self):
         self.text.focus_set()
 
@@ -856,7 +905,7 @@ class EnhancedTextFrame(TextFrame):
         relief="sunken",
         gutter_background="#e0e0e0",
         gutter_foreground="#999999",
-        **text_options
+        **text_options,
     ):
         self._gutter = None
 
@@ -871,7 +920,7 @@ class EnhancedTextFrame(TextFrame):
             horizontal_scrollbar_style=horizontal_scrollbar_style,
             borderwidth=borderwidth,
             relief=relief,
-            **text_options
+            **text_options,
         )
 
         self._recommended_line_length = line_length_margin
@@ -1050,11 +1099,15 @@ class EnhancedTextFrame(TextFrame):
                 delta = 0
 
             if margin_line_visible_col > -1:
-                x = (
-                    get_text_font(self.text).measure((margin_line_visible_col - 1) * "M")
-                    + delta
-                    + self.text["padx"]
-                )
+                try:
+                    x = (
+                        get_text_font(self.text).measure((margin_line_visible_col - 1) * "M")
+                        + delta
+                        + self.text["padx"]
+                    )
+                except TclError:
+                    logger.exception("Could not measure text")
+                    x = -10
             else:
                 x = -10
 
@@ -1072,7 +1125,7 @@ class EnhancedTextFrame(TextFrame):
             ):  # In Python 3.6 you can use tk.EventType.ButtonPress instead of "4"
                 self.text.tag_remove("sel", "1.0", "end")
         except tk.TclError:
-            exception("on_gutter_click")
+            logger.exception("on_gutter_click")
 
     def on_gutter_double_click(self, event=None):
         try:
@@ -1080,7 +1133,7 @@ class EnhancedTextFrame(TextFrame):
             self.text.tag_remove("sel", "1.0", "end")
             self._gutter.tag_remove("sel", "1.0", "end")
         except tk.TclError:
-            exception("on_gutter_click")
+            logger.exception("on_gutter_click")
 
     def on_gutter_motion(self, event=None):
         try:
@@ -1094,7 +1147,7 @@ class EnhancedTextFrame(TextFrame):
             self.text.mark_set("insert", "%s.0" % linepos)
             self.text.focus_set()
         except tk.TclError:
-            exception("on_gutter_motion")
+            logger.exception("on_gutter_motion")
 
     def _vertical_scrollbar_update(self, *args):
         if not hasattr(self, "_vbar"):
@@ -1196,7 +1249,7 @@ def _running_on_x11():
 
 def get_keyboard_language():
     # https://stackoverflow.com/a/42047820/261181
-    if platform.system() != "Windows":
+    if sys.platform != "win32":
         raise NotImplementedError("Can provide keyboard language only on Windows")
 
     import ctypes
@@ -1208,7 +1261,7 @@ def get_keyboard_language():
     klid = user32.GetKeyboardLayout(thread_id)
     # Language ID -> low 10 bits, Sub-language ID -> high 6 bits
     # Extract language ID from KLID
-    lid = klid & (2 ** 16 - 1)
+    lid = klid & (2**16 - 1)
 
     return lid
 
