@@ -1,11 +1,7 @@
 # -*- coding: utf-8 -*-
-import collections
-import logging
 import os
 import platform
-import queue
 import re
-import signal
 import subprocess
 import sys
 import textwrap
@@ -14,9 +10,9 @@ import time
 import tkinter as tk
 import tkinter.font
 import traceback
-import warnings
+from logging import getLogger
 from tkinter import filedialog, messagebox, ttk
-from typing import Callable, List, Optional, Tuple, Union  # @UnusedImport
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union  # @UnusedImport
 
 from _tkinter import TclError
 
@@ -33,13 +29,25 @@ from thonny.tktextext import TweakableText
 
 PARENS_REGEX = re.compile(r"[\(\)\{\}\[\]]")
 
-logger = logging.getLogger(__name__)
+logger = getLogger(__name__)
 
 
 class CommonDialog(tk.Toplevel):
-    def __init__(self, master=None, cnf={}, **kw):
-        super().__init__(master=master, cnf=cnf, **kw)
-        self.bind("<FocusIn>", self._unlock_on_focus_in, True)
+    def __init__(self, master=None, **kw):
+        assert master
+        super().__init__(master=master, class_="Thonny", **kw)
+        self.withdraw()  # remain invisible until size calculations are done
+
+        # TODO: Is it still required ?
+        # self.bind("<FocusIn>", self._unlock_on_focus_in, True)
+
+        # https://bugs.python.org/issue43655
+        if self._windowingsystem == "aqua":
+            self.tk.call("::tk::unsupported::MacWindowStyle", "style", self, "moveableModal", "")
+        elif self._windowingsystem == "x11":
+            self.wm_attributes("-type", "dialog")
+
+        self.parent = master
 
     def _unlock_on_focus_in(self, event):
         if not self.winfo_ismapped():
@@ -48,22 +56,57 @@ class CommonDialog(tk.Toplevel):
             if focussed_widget:
                 focussed_widget.focus_set()
 
-    def get_padding(self):
-        return ems_to_pixels(2)
+    def get_large_padding(self):
+        return ems_to_pixels(1.5)
 
-    def get_internal_padding(self):
-        return self.get_padding() // 4
+    def get_medium_padding(self):
+        return ems_to_pixels(1)
+
+    def get_small_padding(self):
+        return ems_to_pixels(0.6)
+
+    def set_initial_focus(self, node=None) -> bool:
+        if node is None:
+            node = self
+
+        if isinstance(
+            node,
+            (
+                ttk.Entry,
+                ttk.Combobox,
+                ttk.Treeview,
+                tk.Text,
+                ttk.Notebook,
+                ttk.Button,
+                tk.Listbox,
+            ),
+        ):
+            node.focus_set()
+            return True
+
+        else:
+            for child in node.winfo_children():
+                if self.set_initial_focus(child):
+                    return True
+
+        return False
 
 
 class CommonDialogEx(CommonDialog):
-    def __init__(self, master=None, cnf={}, **kw):
-        super().__init__(master=master, cnf=cnf, **kw)
+    def __init__(self, master=None, **kw):
+        super().__init__(master=master, **kw)
+
+        self.rowconfigure(0, weight=1)
+        self.columnconfigure(0, weight=1)
 
         # Need to fill the dialog with a frame to gain theme support
         self.main_frame = ttk.Frame(self)
-        self.main_frame.grid(row=0, column=0, sticky="nsew")
-        self.columnconfigure(0, weight=1)
-        self.rowconfigure(0, weight=1)
+
+        # ipady doesn't work right, at least on Linux (it only applies to the first gridded child)
+        # therefore only providing common padding for left and right edges
+        self.main_frame.grid(row=0, column=0, sticky="nsew", ipadx=self.get_large_padding())
+        self.main_frame.rowconfigure(0, weight=1)
+        self.main_frame.columnconfigure(0, weight=1)
 
         self.bind("<Escape>", self.on_close, True)
         self.protocol("WM_DELETE_WINDOW", self.on_close)
@@ -86,7 +129,7 @@ class QueryDialog(CommonDialogEx):
         self.var = tk.StringVar(value=initial_value)
         self.result = None
 
-        margin = self.get_padding()
+        margin = self.get_large_padding()
         spacing = margin // 2
 
         self.title(title)
@@ -486,13 +529,13 @@ class ClosableNotebook(ttk.Notebook):
             self._popup_index = index
             self.tab_menu.tk_popup(*self.winfo_toplevel().winfo_pointerxy())
         except Exception:
-            logging.exception("Opening tab menu")
+            logger.exception("Opening tab menu")
 
     def _close_tab_from_menu(self):
         self.close_tab(self._popup_index)
 
     def _close_other_tabs(self):
-        self.close_tabs(self._popup_index)
+        self.close_tabs(except_index=self._popup_index)
 
     def close_tabs(self, except_index=None):
         for tab_index in reversed(range(len(self.winfo_children()))):
@@ -692,7 +735,7 @@ class TreeFrame(ttk.Frame):
         show_statusbar=False,
         borderwidth=0,
         relief="flat",
-        **tree_kw
+        **tree_kw,
     ):
         ttk.Frame.__init__(self, master, borderwidth=borderwidth, relief=relief)
         # http://wiki.tcl.tk/44444#pagetoc50f90d9a
@@ -709,7 +752,7 @@ class TreeFrame(ttk.Frame):
             columns=columns,
             displaycolumns=displaycolumns,
             yscrollcommand=self.vert_scrollbar.set,
-            **tree_kw
+            **tree_kw,
         )
         self.tree["show"] = "headings"
         self.tree.grid(row=0, column=0, sticky=tk.NSEW)
@@ -936,16 +979,40 @@ class AutoScrollbar(SafeScrollbar):
     # works if you use the grid geometry manager.
 
     def __init__(self, master=None, **kw):
+        self.hide_count = 0
+        self.gridded = False
         super().__init__(master=master, **kw)
 
     def set(self, first, last):
         if float(first) <= 0.0 and float(last) >= 1.0:
-            self.grid_remove()
-        elif float(first) > 0.001 or float(last) < 0.009:
+            # Need to accept 1 automatic hide, otherwise even narrow files
+            # get horizontal scrollbar
+            if self.gridded and self.hide_count < 2:
+                self.grid_remove()
+        elif float(first) > 0.001 or float(last) < 0.999:
             # with >0 and <1 it occasionally made scrollbar wobble back and forth
-            self.grid()
+            if not self.gridded:
+                self.grid()
         ttk.Scrollbar.set(self, first, last)
 
+    def grid(self, *args, **kwargs):
+        super().grid(*args, **kwargs)
+        self.gridded = True
+
+    def grid_configure(self, *args, **kwargs):
+        super().grid_configure(*args, **kwargs)
+        self.gridded = True
+
+    def grid_remove(self):
+        super().grid_remove()
+        self.gridded = False
+        self.hide_count += 1
+
+    def grid_forget(self):
+        super().grid_forget()
+        self.gridded = False
+        self.hide_count += 1
+
     def pack(self, **kw):
         raise tk.TclError("cannot use pack with this widget")
 
@@ -1393,22 +1460,60 @@ def create_boolean_var(value, modification_listener=None) -> EnhancedBooleanVar:
     return EnhancedBooleanVar(None, value, None, modification_listener)
 
 
-def shift_is_pressed(event_state):
-    # http://infohost.nmt.edu/tcc/help/pubs/tkinter/web/event-handlers.html
+def shift_is_pressed(event: tk.Event) -> bool:
+    # https://tkdocs.com/shipman/event-handlers.html
     # http://stackoverflow.com/q/32426250/261181
-    return event_state & 0x0001
+    return event.state & 0x0001
 
 
-def caps_lock_is_on(event_state):
-    # http://infohost.nmt.edu/tcc/help/pubs/tkinter/web/event-handlers.html
+def caps_lock_is_on(event: tk.Event) -> bool:
+    # https://tkdocs.com/shipman/event-handlers.html
     # http://stackoverflow.com/q/32426250/261181
-    return event_state & 0x0002
+    return event.state & 0x0002
 
 
-def control_is_pressed(event_state):
-    # http://infohost.nmt.edu/tcc/help/pubs/tkinter/web/event-handlers.html
+def control_is_pressed(event: tk.Event) -> bool:
+    # https://tkdocs.com/shipman/event-handlers.html
     # http://stackoverflow.com/q/32426250/261181
-    return event_state & 0x0004
+    return event.state & 0x0004
+
+
+def alt_is_pressed_without_char(event: tk.Event) -> bool:
+    # https://tkdocs.com/shipman/event-handlers.html
+    # http://stackoverflow.com/q/32426250/261181
+    # https://bugs.python.org/msg268429
+    if event.char:
+        return False
+
+    if running_on_windows():
+        return event.state & 0x20000
+    elif running_on_mac_os():
+        # combinations always produce a char or are consumed by the OS
+        return False
+    else:
+        return event.state & 0x0010
+
+
+def command_is_pressed(event: tk.Event) -> bool:
+    # https://tkdocs.com/shipman/event-handlers.html
+    # http://stackoverflow.com/q/32426250/261181
+    if not running_on_mac_os():
+        return False
+    return event.state & 0x0008
+
+
+def get_hyperlink_cursor() -> str:
+    if running_on_mac_os():
+        return "pointinghand"
+    else:
+        return "hand2"
+
+
+def get_beam_cursor() -> str:
+    if running_on_mac_os() or running_on_windows():
+        return "ibeam"
+    else:
+        return "xterm"
 
 
 def sequence_to_event_state_and_keycode(sequence: str) -> Optional[Tuple[int, int]]:
@@ -1438,7 +1543,7 @@ def sequence_to_event_state_and_keycode(sequence: str) -> Optional[Tuple[int, in
         return None
 
     event_state = 0
-    # http://infohost.nmt.edu/tcc/help/pubs/tkinter/web/event-handlers.html
+    # https://tkdocs.com/shipman/event-handlers.html
     # https://stackoverflow.com/questions/32426250/python-documentation-and-or-lack-thereof-e-g-keyboard-event-state
     for modifier in modifiers:
         if modifier == "shift":
@@ -1506,53 +1611,50 @@ def remove_line_numbers(s):
     return textwrap.dedent(("\n".join(cleaned_lines)) + "\n")
 
 
-def center_window(win, master=None):
-    # for backward compat
-    return assign_geometry(win, master)
+# Place a toplevel window at the center of parent or screen
+# It is a Python implementation of ::tk::PlaceWindow.
+# Copied and adapted from tkinter.simpledialog of Python 3.10.2
+def _place_window(w, parent=None, width=None, height=None):
+    w.wm_withdraw()  # Remain invisible while we figure out the geometry
+    w.update_idletasks()  # Actualize geometry information
 
+    minwidth = width or w.winfo_reqwidth()
+    minheight = height or w.winfo_reqheight()
+    maxwidth = w.winfo_vrootwidth()
+    maxheight = w.winfo_vrootheight()
+    if parent is not None and parent.winfo_ismapped():
+        logger.info(
+            f"Parent y: {parent.winfo_y()}, rooty: {parent.winfo_rooty()}, vrooty: {parent.winfo_vrooty()}"
+        )
+        x = parent.winfo_rootx() + (parent.winfo_width() - minwidth) // 2
+        y = parent.winfo_y() + (parent.winfo_height() - minheight) // 2
+        vrootx = w.winfo_vrootx()
+        vrooty = w.winfo_vrooty()
+        x = min(x, vrootx + maxwidth - minwidth)
+        x = max(x, vrootx)
+        y = min(y, vrooty + maxheight - minheight)
+        # don't allow the dialog go higher than parent. This way the title bar remains visible.
+        y = max(y, vrooty, parent.winfo_y())
+        if w._windowingsystem == "aqua":
+            # Avoid the native menu bar which sits on top of everything.
+            y = max(y, ems_to_pixels(2))
 
-def assign_geometry(win, master=None, min_left=0, min_top=0):
-    if master is None:
-        master = tk._default_root
-
-    size = get_workbench().get_option(get_size_option_name(win))
-    if size:
-        width, height = size
-        saved_size = True
+        if y + minheight > maxheight:
+            logger.debug("Aligning top with parent (%s vs %s)", y + minheight, maxheight)
+            y = parent.winfo_y()
     else:
-        fallback_width = 600
-        fallback_height = 400
-        # need to wait until size is computed
-        # (unfortunately this causes dialog to jump)
-        if getattr(master, "initializing", False):
-            # can't get reliable positions when main window is not in mainloop yet
-            width = fallback_width
-            height = fallback_height
-        else:
-            if not running_on_linux():
-                # better to avoid in Linux because it causes ugly jump
-                win.update_idletasks()
-            # looks like it doesn't take window border into account
-            width = win.winfo_width()
-            height = win.winfo_height()
+        x = (w.winfo_screenwidth() - minwidth) // 2
+        y = (w.winfo_screenheight() - minheight) // 2
 
-            if width < 10:
-                # ie. size measurement is not correct
-                width = fallback_width
-                height = fallback_height
-
-        saved_size = False
-
-    left = master.winfo_rootx() + master.winfo_width() // 2 - width // 2
-    top = master.winfo_rooty() + master.winfo_height() // 2 - height // 2
-
-    left = max(left, min_left)
-    top = max(top, min_top)
-
-    if saved_size:
-        win.geometry("%dx%d+%d+%d" % (width, height, left, top))
+    w.wm_maxsize(maxwidth, maxheight)
+    if width and height:
+        geometry = "%dx%d+%d+%d" % (width, height, x, y)
     else:
-        win.geometry("+%d+%d" % (left, top))
+        geometry = "+%d+%d" % (x, y)
+
+    logger.info(f"Placing {w} with geometry {geometry}")
+    w.wm_geometry(geometry)
+    w.wm_deiconify()  # Become visible at the desired location
 
 
 class WaitingDialog(CommonDialog):
@@ -1690,6 +1792,7 @@ class ChoiceDialog(CommonDialogEx):
         choices=[],
         initial_choice_index=None,
     ) -> None:
+        self.result = None
         super().__init__(master=master)
 
         self.title(title)
@@ -1702,7 +1805,7 @@ class ChoiceDialog(CommonDialogEx):
         question_label.grid(row=row, column=0, columnspan=2, sticky="w", padx=20, pady=20)
         row += 1
 
-        self.var = tk.StringVar("")
+        self.var = tk.StringVar(value="")
         if initial_choice_index is not None:
             self.var.set(choices[initial_choice_index])
         for choice in choices:
@@ -1874,7 +1977,7 @@ def _get_dialog_provider():
 
 
 def asksaveasfilename(**options):
-    # https://tcl.tk/man/tcl8.6/TkCmd/getSaveFile.htm
+    # https://tcl.tk/man/tcl8.6/TkCmd/getOpenFile.htm
     _check_dialog_parent(options)
     return _get_dialog_provider().asksaveasfilename(**options)
 
@@ -1956,9 +2059,6 @@ class _ZenityDialogProvider:
         if not filename:
             return None
 
-        if "defaultextension" in options and "." not in os.path.basename(filename):
-            filename += options["defaultextension"]
-
         return filename
 
     @classmethod
@@ -2004,9 +2104,9 @@ class _ZenityDialogProvider:
         if result.returncode == 0:
             return result.stdout.strip()
         else:
-            # TODO: log problems
-            print(result.stderr, file=sys.stderr)
-            # could check stderr, but it may contain irrelevant warnings
+            logger.warning(
+                "Zenity returned code %r and stderr %r", result.returncode, result.stderr
+            )
             return None
 
 
@@ -2046,7 +2146,7 @@ def handle_mistreated_latin_shortcuts(registry, event):
     # because at least on Windows, Ctrl-shortcuts' state
     # has something extra
     simplified_state = 0x04
-    if shift_is_pressed(event.state):
+    if shift_is_pressed(event):
         simplified_state |= 0x01
 
     # print(simplified_state, event.keycode)
@@ -2059,7 +2159,7 @@ def handle_mistreated_latin_shortcuts(registry, event):
                     handler()
 
 
-def show_dialog(dlg, master=None, geometry=True, min_left=0, min_top=0):
+def show_dialog(dlg, master=None, width=None, height=None):
     if getattr(dlg, "closed", False):
         return
 
@@ -2070,26 +2170,31 @@ def show_dialog(dlg, master=None, geometry=True, min_left=0, min_top=0):
 
     get_workbench().event_generate("WindowFocusOut")
     # following order seems to give most smooth appearance
-    focused_widget = master.focus_get()
-    dlg.transient(master.winfo_toplevel())
+    old_focused_widget = master.focus_get()
+    if master.winfo_toplevel().winfo_viewable():
+        dlg.transient(master.winfo_toplevel())
 
-    if geometry:
-        # dlg.withdraw() # unfortunately inhibits size calculations in assign_geometry
-        if isinstance(geometry, str):
-            dlg.geometry(geometry)
-        else:
-            assign_geometry(dlg, master, min_left, min_top)
-        # dlg.wm_deiconify()
+    saved_size = get_workbench().get_option(get_size_option_name(dlg))
+    if saved_size:
+        width = min(max(saved_size[0], ems_to_pixels(10)), ems_to_pixels(500))
+        height = min(max(saved_size[1], ems_to_pixels(8)), ems_to_pixels(300))
+
+    _place_window(dlg, master, width=width, height=height)
 
     dlg.lift()
-    dlg.focus_set()
+    dlg.wait_visibility()
 
     try:
         dlg.grab_set()
     except TclError as e:
-        print("Can't grab:", e, file=sys.stderr)
+        logger.warning("Can't grab: %s", e)
 
-    master.winfo_toplevel().wait_window(dlg)
+    dlg.update_idletasks()
+    dlg.focus_set()
+    if hasattr(dlg, "set_initial_focus"):
+        dlg.set_initial_focus()
+
+    dlg.wait_window(dlg)
     dlg.grab_release()
     master.winfo_toplevel().lift()
     master.winfo_toplevel().focus_force()
@@ -2097,9 +2202,9 @@ def show_dialog(dlg, master=None, geometry=True, min_left=0, min_top=0):
     if running_on_mac_os():
         master.winfo_toplevel().grab_release()
 
-    if focused_widget is not None:
+    if old_focused_widget is not None:
         try:
-            focused_widget.focus_force()
+            old_focused_widget.focus_force()
         except TclError:
             pass
 
@@ -2229,17 +2334,17 @@ class TextMenu(MenuEx):
         return False
 
 
-def create_url_label(master, url, text=None):
+def create_url_label(master, url, text=None, **kw):
     import webbrowser
 
-    return create_action_label(master, text or url, lambda _: webbrowser.open(url))
+    return create_action_label(master, text or url, lambda _: webbrowser.open(url), **kw)
 
 
 def create_action_label(master, text, click_handler, **kw):
     url_font = tkinter.font.nametofont("TkDefaultFont").copy()
     url_font.configure(underline=1)
     url_label = ttk.Label(
-        master, text=text, style="Url.TLabel", cursor="hand2", font=url_font, **kw
+        master, text=text, style="Url.TLabel", cursor=get_hyperlink_cursor(), font=url_font, **kw
     )
     url_label.bind("<Button-1>", click_handler)
     return url_label
@@ -2249,18 +2354,9 @@ def get_size_option_name(window):
     return "layout." + type(window).__name__ + "_size"
 
 
-def get_default_theme():
-    if running_on_windows():
-        return "Windows"
-    elif running_on_rpi():
-        return "Raspberry Pi"
-    else:
-        return "Enhanced Clam"
-
-
 def get_default_basic_theme():
     if running_on_windows():
-        return "xpnative"
+        return "vista"
     else:
         return "clam"
 
@@ -2268,7 +2364,7 @@ def get_default_basic_theme():
 EM_WIDTH = None
 
 
-def ems_to_pixels(x):
+def ems_to_pixels(x: float) -> int:
     global EM_WIDTH
     if EM_WIDTH is None:
         EM_WIDTH = tkinter.font.nametofont("TkDefaultFont").measure("m")
@@ -2319,5 +2415,89 @@ def add_messagebox_parent_checker():
         setattr(messagebox, name, wrap_with_parent_checker(fun))
 
 
+def replace_unsupported_chars(text: str) -> str:
+    if get_tk_version_info() < (8, 6, 12):
+        # can crash with emojis
+        return "".join(c if c < "\U00010000" else "□" for c in text)
+    else:
+        return text
+
+
+def windows_known_extensions_are_hidden() -> bool:
+    assert running_on_windows()
+    import winreg
+
+    reg_key = winreg.OpenKey(
+        winreg.HKEY_CURRENT_USER,
+        r"SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Advanced",
+        0,
+        winreg.KEY_READ,
+    )
+    try:
+        return winreg.QueryValueEx(reg_key, "HideFileExt")[0] == 1
+    finally:
+        reg_key.Close()
+
+
+class MappingCombobox(ttk.Combobox):
+    def __init__(self, master, mapping=None, **kw):
+        super().__init__(master, **kw)
+
+        if mapping is None:
+            mapping = {}
+
+        self.mapping: Dict[str, Any]
+        self.set_mapping(mapping)
+        self.mapping_desc_variable = tk.StringVar(value="")
+        self.configure(textvariable=self.mapping_desc_variable)
+
+        self.state(["!disabled", "readonly"])
+
+    def set_mapping(self, mapping: Dict[str, Any]):
+        self.mapping = mapping
+        self["values"] = list(mapping)
+
+    def get_selected_value(self) -> Any:
+        desc = self.mapping_desc_variable.get()
+        return self.mapping.get(desc, None)
+
+    def select_value(self, value) -> None:
+        for desc in self.mapping:
+            if self.mapping[desc] == value:
+                self.set(desc)
+
+    def select_none(self) -> None:
+        self.mapping_desc_variable.set("")
+
+
+class AdvancedLabel(ttk.Label):
+    def __init__(self, master, **kw):
+        self._default_font = tkinter.font.nametofont("TkDefaultFont")
+        self._url_font = self._default_font.copy()
+        self._url_font.configure(underline=1)
+        self._url = None
+        super().__init__(master, **kw)
+        self.bind("<Button-1>", self._on_click, True)
+
+    def set_url(self, url: Optional[str]) -> None:
+        if self._url == url:
+            return
+
+        self._url = url
+        if url:
+            self.configure(style="Url.TLabel", cursor=get_hyperlink_cursor(), font=self._url_font)
+        else:
+            self.configure(style="TLabel", cursor="", font=self._default_font)
+
+    def get_url(self) -> Optional[str]:
+        return self._url
+
+    def _on_click(self, *event):
+        if self._url:
+            import webbrowser
+
+            webbrowser.open(self._url)
+
+
 if __name__ == "__main__":
-    root = tk.Tk()
+    print(windows_known_extensions_are_hidden())
